/* seqpp/Coder.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*!
  \file
  \author Vincent Miele
  \brief Coder

   07/2004

   Coded abstract representation of a word
*/
#ifndef CODER_H
#define CODER_H

#include <vector>
#include <iostream>

using namespace std;

/**
 * \ingroup General_Use_Sequence
 */

/*!
  \internal \brief  Coded abstract representation of a word

  Management of a code related to a word 
*/

class Coder
{
 private:

  //! markov order
  short _order;
  //! size of the alphabet
  short _size;

  //! power of size
  long * _pow;
  //! _jump[i] is the cumulative jump to access to codes of word of size > i
  long * _jump;
  //! _pow_r[i][pos] is the value of i on position pos in a word coded in base _size
  long ** _pow_r;
  
  //! current vector of code
  vector<long> _vect;
  
  //! length of the current coded word
  short compute_length() const;
  
 public:
  //! code
  long _code;
  //! size of the coded word 
  short _sizeword;
  
  //! constructor with the markovian order+1 == length of words and the size of the alphabet
  Coder( short order, short size );

  /*! constructor with the markovian order+1 == length of words and the size of the alphabet
    first_code is an initial word of size sizeword */
  Coder( short order, short size,
	 long first_code, short sizeword );
  
  //! empty constructor
  Coder( )
  {
    _order = -1; _size = 0; 
    _pow = NULL; _jump = NULL; 
    _pow_r = NULL;
  }  
  
  //! copy constructor
  Coder( const Coder& c );
  
  //! destructor
  ~Coder( );  
  
    
  //! intial code is an initial word of size sizeword
  void init( long code, short sizeword ){
    _code = code;
    _sizeword = sizeword;
  }

  //! intial code is an initial word. The size sizeword is computed.
  void init( long code ){
    _code = code;
    _sizeword = this->compute_length();
  }
  
  //! initial by coding the word X, using only the last _order+1
  void init( const vector<short>& X );

  //! initial by coding the word "X(-_order)...X(-1)X(0)" 
  void init( const short * X );

  //! initial by coding the word "X(-pseudo_order)...X(1)X(0)" 
  void restrict_init( const short * X,  short pseudo_order );

  //! put code down to -1
  void clear(){
    _code = -1;
    _sizeword = 0;
  }

  
  //! add a letter at the end of the coded-word
  void push_back( short i ){
    _code = (_code+1)*_size + i;
    _sizeword++;
  }
  //! return the code obtained by adding a letter at the end of the coded-word but don't touch _code
  long virtual_push_back( short i ) const{
    return (_code+1)*_size + i;
  }


  //! add a letter at the beginning of the coded-word
  void push_front( short i ){
    _code += _pow[_sizeword] + _pow_r[i][_order-_sizeword];
    _sizeword++;
  }
  //! return the code obtained by adding a letter at the beginning of the coded-word but don't touch _code
  long virtual_push_front( short i ) const{
    return (_code + _pow[_sizeword] + _pow_r[i][_order-_sizeword]);
  }

  //! erase the last letter
  void erase_back(){
    _code = _code/_size - 1;
    _sizeword--;
  }
  //! erase the last letter of the coded-word and return it
  short pop_back();
   
          
  //! erase the first letter of the coded-word
  void erase_front(){
    _sizeword--;
    _code = _code - _pow_r[  (_code-_jump[_sizeword])/_pow[_sizeword]  ][ _order-_sizeword ] - _pow[_sizeword];
  }
  //! erase the first letter and return it  
  short pop_front();

  //! checks if coder is empty (no word coded)
  bool empty() const{
    return (_code==-1);
  }

  //! length of the current coded word
  short length() const{
    return _sizeword;
  }

  //! return nb of possible codes == words from a singleton to a order+1-word
  long tell_nbvalue() const{
    return _jump[_order+1];
  }

  //! return the ith jump to codes of word of i (and more than i) letters
  long tell_jump( short i ) const{
    return _jump[i];
  }
  //! return the ith jump to codes of word of _order+1 letters
  long tell_jump( ) const{
    return _jump[_order];
  }
  
  //! allows a const access to the vector jump
  long * get_jump() const{
    return _jump;
  }

   //! returns the vector of the code of words suffixed by the current _code with prefixes of size sizeprefix
  vector<long> & list_suffixed( short sizeprefix );
  //! returns the vector of the code of words suffixed by the current _code with prefixes adapted to (_order+1)-words
  vector<long> & list_suffixed( ){
    return list_suffixed( _order+1 - _sizeword );
  }

  //! returns in a vect the 2 bound-codes between them all (including both of them) codes are prefixed by the current _code with suffixes of size sizesuffix
  vector<long> & bound_prefixed( short sizesuffix );
 //! returns in a vect the 2 bound-codes between them all (including both of them) codes are prefixed by the current _code with suffixes adapted to (_order+1)-words
  vector<long> & bound_prefixed( ){
    return bound_prefixed( _order+1 - _sizeword );
  }
};
#endif
