/* seqpp/MTDMarkov.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
 \file
  \author Vincent Miele
  \brief Mixture Transition Distribution Markov model

   01/2004
*/

#ifndef SEQPP_MTDMARKOV_H
#define SEQPP_MTDMARKOV_H

#include <seqpp/Markov.h>
#include <seqpp/mtd_core.h>

/**
 * \ingroup General_Use_Markov
 */

/*!  \class MTDMarkov seqpp/MTDMarkov.h
  \brief    Mixture Transition Distribution Markov modelling, estimation and simulation.

  MTDMarkov is a Markov object with a different estimation step.
  This object performs the estimation with the Mixture Transition Distribution algorithm 
  and then transfoms, once per phase, the  MTD model in a markovian matrix.
*/
class MTDMarkov : public Markov
{
 protected : 
  //! Parcimonious Context Trees
  mtd_core * _mtdcore;

 public:
  //! Constructor 1 from a SequenceSet or a Sequence
  /*!
    \param  tseq a set of sequences or a sequence for estimation
    \param mkv_order markovian order on the markovian matrix of the MTD
    \param nbseed number of seeds for the EM algorithm
    \param nbiter_max maximum iterations number of the EM algorithm
    \param eps value of the epsilon of the EM algorithm
    \param log true to log the successive likelihood values
  */
  template< class TSeq >
  MTDMarkov( const TSeq & tseq, short mkv_order,
	     short nbseed = NBSEED, int nbiter_max = NBITERMAX, double eps = EPS,
	     bool log = false, bool identical_pi = false )
    :  Markov(  tseq.tell_alphabet_size(), tseq.tell_order(), phase )
  { 
    tseq.count_p_occurencies( ); 
    estimate( tseq.get_p_count(), true, mkv_order, 
	      nbseed, nbiter_max, eps, identical_pi );
  }
 
  //! Constructor 2 from a coded-word count
  /*!
    \param count count of all the coded word(base size) of size order+1 for each phase, for estimation
    \param size alphabet size
    \param mtd_order markovian order for the resulting model
    \param mkv_order markovian order on the markovian matrices used in the MTD
    \param nbseed number of seeds for the EM algorithm
    \param nbiter_max maximum iterations number of the EM algorithm
    \param eps value of the epsilon of the EM algorithm
    \param log true to log the successive likelihood values
  */
  MTDMarkov( unsigned long * * count,
	     short size, 
	     short mtd_order, short mkv_order,
	     short nbseed = NBSEED, int nbiter_max = NBITERMAX, double eps = EPS,
	     bool log = false, bool identical_pi = false )
    :  Markov( size, mtd_order )
  {       
    estimate( count, false, mkv_order, 
	      nbseed, nbiter_max, eps );
  }

  //! performs the estimation [used by each constructor] 
  void estimate( unsigned long * * count, bool decal_required,
		 short mkv_order,
		 short nbseed, int nbiter_max, double eps,
		 bool log, bool identical_pi )
 {   
   _mtdcore = new mtd_core( _size, _order, mkv_order );     
   _nb_param = _mtdcore->nb_params();
   _mtdcore->estimate( count[i], decal_required,
		       nbseed, nbiter_max, eps, log, identical_pi );
   _mtdcore->mtd_to_matrix( _Pi );

   delete _mtdcore;
 }
 
//! Destructor
 ~MTDMarkov(){} 
};
#endif
