/* seqpp/Markov.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
 \file
  \author David Robelin - Vincent Miele
  \brief Markov model

   01/2004
*/

#ifndef SEQPP_MARKOV_H
#define SEQPP_MARKOV_H

#include <seqpp/PhasedMarkov.h>
#include <seqpp/arnoldi.h>

/**
 * \ingroup General_Use_Markov
 */

/*! \class Markov seqpp/Markov.h
 \brief  Markov modelling, estimation and simulation.

This is a special case of a phased Markov [PhasedMarkov] model when only one phase is considered.
*/
class Markov : public PhasedMarkov
{
 public :
 
  //! Constructor 1 : read a configuration file 
  /*
    \param markov_file description of an input model in a file (generated by print method) 
    \param calc_rank calculus of the convergence rank if true
    Exple of file:
    \code
    # 4 <- Order of the phased Markov chain<br>
    # 3 <- Phase<br>
    # 4 <- Alphabet size<br>
    # 10 steps <- Convergence to the stationnary distribution (!!Optionnal!!)<br>
    # Transition matrix:<br>
    ...<br>
    # Stationnary Probability: (!!Optionnal!!)<<br>
    ...
    \endcode
  */
  Markov( const char * ConfFile, 
	  bool calc_rank = false );
  //! Constructor 2 : Estimate the transition matrices on the sequences of seqset
  /*!
    \param seqset set of sequences for estimation
    \param calc_rank calculus of the convergence rank if true
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element
  */
  Markov( const SequenceSet & seqset,
	  bool calc_rank = false,
	  const string& prior_alpha_file = string() );

  //! Constructor 3 : Estimate the transition matrices on the sequence seq
  /*!
    \param seq sequence for estimation   
    \param calc_rank calculus of the convergence rank if true
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element
  */
  Markov( const Sequence & seq,
	  bool calc_rank = false,
	  const string& prior_alpha_file = string() );

  //! Constructor 4 : Copy constructor
  Markov( const Markov & );
  
  //! Constructor 5 : Default constructor
  Markov();

 //! Constructor 6 : Minimal Constructor
  /*!
    Initialises the constants of the model but not the matrix nor the stat law
    \param size alphabet size
    \param order markovian order
    \param alloc true for matrices memory allocation
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element 
  */
  Markov( short size, short order, bool alloc=true,
	  const string& prior_alpha_file = string() ):PhasedMarkov(size,order,1,alloc, prior_alpha_file) 
  {//cout<<"Markov constr. 6"<<endl;
    if (alloc){
      _Pi=_Pis[0];_container=_containers[0];
    }
    else{
      _Pi = NULL; _container=NULL;
    }
    _Mu=NULL;_PowPi=NULL;
  }

  //! Constructor 7 : Creation of a "mixed" Markov chain M = p*M1 + (1-p)*M2 */
  /*!
    \param M1 first Markov chain object
    \param M2 second Markov chain object
    \param p weight of M1 in the resulting M(with 0<=p<=1)
  */
  Markov(const Markov &M1, const Markov &M2,const float p);

 //! Constructor 8 : random markov matrices 
  /*!
    \param r gsl random generator
    \param size alphabet size
    \param order markovian order
    \param calc_rank calculus of the convergence rank if true
    <br>GSL use exple:<br> 
    \code
    const gsl_rng_type * T;
    // Choice a default generator and seed
    // from environment variables
    gsl_rng_env_setup();
    // New created instance of the generator
    T = gsl_rng_default;
    gsl_rng * r = gsl_rng_alloc (T);
    // Initialize/Seeds the random number generator
    gsl_rng_set( r, (long)getpid() );
    \endcode    
  */
  Markov( const gsl_rng * r,
	  short size, short order,
	  bool calc_rank = false)
    : PhasedMarkov( r, size, order, 1, calc_rank ){
    _Pi=_Pis[0];_container=_containers[0];_Mu=NULL;_PowPi=NULL;
  };

  //! Constructor 9 : Estimate the transition matrices on a word-count
  /*!  
    \param count count of all the coded word(base size) of size order+1 for estimation
    \param size alphabet size
    \param order markovian order
    \param calc_rank calculus of the convergence rank if true
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element
  */
  Markov( unsigned long * count,
	  short size, short order,
	  const string& prior_alpha_file = string(),
	  bool calc_rank = false );

  //!Destructor
  virtual ~Markov();

  //---------------------------------------
  //! Estimate the transition matrices on the sequence/sequenceset tseq
  /*!
    \param tseq sequence/sequenceset for estimation
    \param beg begin position in sequence(s) if subsequences 
    \param end end position in sequence(s) if subsequences  
    \param calc_rank calculus of the convergence rank if true
  */
  template <class TSeq>
  void estimate( const TSeq & tseq,
		 unsigned long beg, unsigned long end,
		 bool calc_rank ) {
    this->PhasedMarkov::estimate( tseq, 1,0, beg, end, calc_rank );
  }

  //! Estimate the transition matrices from a word count
  /*! 
    \param count count of all the coded word(base size) of size order+1, for estimation  
    \param decal_required necessary when using a count of word from 1-word to (_order+1)-word
    \param calc_rank calculus of the convergence rank if true 
  */
  void estimate( unsigned long * count, bool decal_required, bool calc_rank= false ){
    this->PhasedMarkov::estimate( &count, decal_required, calc_rank );
  }

    //! Estimate the transition matrices from a file containing the count
  /*! 
    \param count_file file containing the count in the adapted format
    \param calc_rank calculus of the convergence rank if true 
  */
  void estimate( const string& count_file, bool calc_rank= false )
  {
    this->PhasedMarkov::estimate( count_file, calc_rank );
  }


  //---------------------------------------
  //! access to the markov matrix
  const double * markov_matrix() const{
    return PhasedMarkov::markov_matrix(0);
  } 

  //! draw at random the markov matrix 
  /*!
    \param r gsl random generator

    GSL use exple:<br> 
    \code
    const gsl_rng_type * T;
    // Choice a default generator and seed
    // from environment variables
    gsl_rng_env_setup();
    // New created instance of the generator
    T = gsl_rng_default;
    gsl_rng * r = gsl_rng_alloc (T);
    // Initialize/Seeds the random number generator
    gsl_rng_set( r, (long)(time( NULL )) );
    ...
    ...
    gsl_rng_free( r );
    \endcode    
  */
  void draw_markov_matrix(const  gsl_rng * r){
    PhasedMarkov::draw_markov_matrices(r);
  }

  //! free the memory allocated for markov matrix
  void free_markov_matrix(){
    PhasedMarkov::free_markov_matrices();
  }
  
  //! Compute the stationnary laws
  /*
    \param force "true" to force the calculation, even if the law already exists. Default => "false". 
  */ 
  void compute_stat_law( bool force=false ){
    PhasedMarkov::compute_stat_laws( force );
    _Mu = _Mus[0];
  }  
  //! free the memory allocated for stationnary law
  void free_stat_law(){
    PhasedMarkov::free_stat_laws();
    _Mu = NULL;
  }
  //! access to the stationnary distrib
  const double * stat_law() const{
    return PhasedMarkov::stat_law(0);
  }
  //--------------------------------------- 
  //! Computes the rank of convergence of the Markov Chain
  virtual int compute_rank(); 
  
  //! Initialisation the _rank powers of the markov matrix
  void compute_power();
   
  //! free the memory allocated for the power
  int free_power();

  //! transition from word w1 to word w2 in step steps
  /*!
    -a Translator object is required to process string to int, use link_to_translator-
    if step > _rank, give stat_law(w2). You must have w1, w2 of length order
    \param w1 first word (of length order) as a string
    \param w2 second word (of length order) as a string
  */
  // double proba_step( const string & w1, const string & w2, int step ) const;
 //! transition from word w1 to word w2 in step steps
  /*!
    if step > _rank, give stat_law(w2). You must have w1, w2  of length order
    \param w1 first word (of length order) as a vector of short
    \param w2 second word (of length order) as a vector of short
  */
  //double proba_step( const vector<short> & w1, const vector<short> & w2, int step ) const;

  //! transition from word w1 to word w2 in step steps
  /*!
    if step > _rank, give stat_law(w2). You must have w1-jump, w2-jump < _nMu
    \param w1 first word (of length order) as a Sequence-coded-like integer (see Sequence)
    \param w2 second word (of length order) as a Sequence-coded-like integer (see Sequence)
    \param step number of steps between w1 and w2
  */
  double proba_step( long w1, long w2, int step ){
    compute_rank();
    compute_power();
      if (step<=_rank)
	return(_PowPi[step-1][w1][w2]);
      else 
	return(_Mu[w2]);
  }
   
  //---------------------------------------
  /*! _Pi == NULL ?*/
  bool isPi() const { return(_Pi != NULL);};
  /*! _PowPi == NULL ?*/
  bool isPow() const { return(_PowPi != NULL);};
  /*! _Mu == NULL ?*/
  bool isMu()  const { return(_Mu != NULL);};

 //! () operator for Markov matrix Pi elements
  /*!
    \param index index of the word
  */
  double & operator() (int index){ 
    return(_Pi[index]); 
  }
  //! Access to stationnary vector Mu elements
  /*!
    \param index index of the word
  */
  double Mu(int index) const { 
    return(_Mu[index]); 
  }

protected :
  
  //! Pointer to "Matrix" (in a vector format) of transition probabilities (=_Pis[0])
  double *_Pi;
  //! Container of "Matrix" (in a vector format) of transition probabilities (=_containers[0])
  double *_container;
  
  //!Vector of stationnary probabilities  (=_Mus[0])
  double *_Mu; 
  
  //!Power of the _Pi matrix
  double ***_PowPi;
}; 
#endif/*SEQPP_MARKOV_H*/

