/* seqpp/PMarkov.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
 \file
  \author Vincent Miele
  \brief  Parcimonious Markov model

   01/2004
*/

#ifndef SEQPP_PMARKOV_H
#define SEQPP_PMARKOV_H

#include <seqpp/Markov.h>
#include <seqpp/PhasedPMarkov.h>
#include <seqpp/pmm_tree.h>

/**
 * \ingroup General_Use_Markov
 */

/*!  \class PMarkov seqpp/PMarkov.h
  \brief  Parcimonious Markov modelling, estimation and simulation.
  
  This is a special case of a phased Parcimonious Markov [PhasedPMarkov] model when only one phase is considered, and also a Markov object.
*/

class PMarkov :  public Markov
{
 protected :
  pmm_forest * _p_f;

 public:
  //! Constructor 1 from a SequenceSet
  /*!
    \param p associated partition  
    \param  seqset a set of sequences for estimation
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element
    \param motif_prior activate a weight-function on the priors, weight proportionnal on each motif in the tree
    \param penalty penalty on the leaves number, by default 0  
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  PMarkov( Partition& p,
	   const SequenceSet & seqset,
	   const string& prior_alpha_file = string(),
	   bool motif_prior = true,
	   double penalty = 0.,
	   const string & xmlfile = string() )
    :  Markov( seqset.tell_alphabet_size(), seqset.tell_order() )
  { 
    seqset.count_occurencies(); 
    if (!this->load_prior_alpha( prior_alpha_file ))
      this->default_prior_alpha();
    _p_f = new pmm_forest( p, _size, _order, _prior_alpha[0], motif_prior,  penalty );
    select( seqset.get_count(), true, seqset.get_translator(), xmlfile );
  }

  //! Constructor 2 from a Sequence
  /*!
    \param p associated partition   
    \param  seq sequence for estimation
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element
    \param motif_prior activate a weight-function on the priors, weight proportionnal on each motif in the tree
    \param penalty penalty on the leaves number, by default 0  
    \param trans a Translator is required only for the xml saving
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  PMarkov( Partition& p,
	   const Sequence & seq, 
	   const string& prior_alpha_file = string(),
	   bool motif_prior = true,
	   double penalty = 0.,
	   const Translator & trans = Translator(),
	   const string & xmlfile = string() )
    : Markov(  seq.tell_alphabet_size(), seq.tell_order() )
  { 
    seq.count_occurencies();
    if (!this->load_prior_alpha( prior_alpha_file ))
      this->default_prior_alpha();
    _p_f = new pmm_forest( p, _size, _order, _prior_alpha[0], motif_prior, penalty ); 
    select( seq.get_count(), true, trans, xmlfile );
  }


   //! Constructor 3 from a coded-word count
  /*!
    \param p associated partition  
    \param count count of all the coded word(base size) of size order+1 for each phase, for estimation
    \param size alphabet size
    \param order markovian order associated to the word count
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element
    \param motif_prior activate a weight-function on the priors, weight proportionnal on each motif in the tree
    \param penalty penalty on the leaves number, by default 0  
    \param trans a Translator is required only for the xml saving
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  PMarkov( Partition& p,
	   unsigned long * count,
	   short size, short order,
	   const string& prior_alpha_file = string(),
	   bool motif_prior = true,
	   double penalty = 0., 	   
	   const Translator & trans = Translator(),	   
	   const string & xmlfile = string() )
    : Markov( size, order )    
  {
    if (!this->load_prior_alpha( prior_alpha_file ))
      this->default_prior_alpha();
    _p_f = new pmm_forest( p, _size, _order, _prior_alpha[0], motif_prior, penalty );  
    select( count, false, trans, xmlfile );
  }

    //! Constructor 4 from a file containing coded-word count
    /*!
      \param part associated partition
      \param count_file file with count
      \param size alphabet size
      \param order markovian order associated to the word count
      \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element
      \param motif_prior activate a weight-function on the priors, weight proportionnal on each motif in the tree
      \param penalty penalty on the leaves number, by default 0  
      \param trans a Translator is required only for the xml saving
      \param xmlfile xmlfile for tree saving (if xml2 activated)
    */
    PMarkov( Partition& part,
	     const string & count_file,
	     short size, short order,
	     const string& prior_alpha_file = string(),
	     bool motif_prior = true,
	     double penalty = 0.,
	     const Translator & trans = Translator(),
	     const string & xmlfile = string() )
      :  Markov( size, order )
      { 	
	 unsigned long * count = new unsigned long[_nPi];    
	if (! this->load_count( count_file, &count )){
	  cerr<<"Bad count file"<<endl;
	  exit(1);
	}
	if (!this->load_prior_alpha( prior_alpha_file ))
	  this->default_prior_alpha();
	_p_f = new pmm_forest( part, _size, _order, _prior_alpha[0], motif_prior, penalty );      
	select( count, false, trans, xmlfile );
	
	delete [] count;
      }

  //! Basic Constructor 5. No estimation.
  /*! 
    \param part associated partition       
    \param size alphabet size
    \param order markovian order associated to the word count
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element
    \param motif_prior activate a weight-function on the priors, weight proportionnal on each motif in the tree
    \param penalty penalty on the leaves number, by default 0 
    \param alloc true for matrices memory allocation
  */
  PMarkov( Partition& part,
	   short size,
	   short order,
	   const string& prior_alpha_file = string(),
	   bool motif_prior = true,
	   double penalty = 0.,
	   bool alloc = true )
    : Markov( size, order, alloc )
  {
    if (!this->load_prior_alpha( prior_alpha_file ))
      this->default_prior_alpha();
    _p_f = new pmm_forest( part, _size, _order, _prior_alpha[0], motif_prior, penalty );
  }


  //! performs the A Posteriori Maximisation
  /*! 
    \param count count of all the coded word(base size) of size order+1 for each phase, for estimation
    \param decal_required necessary when using a count of word from 1-word to (_max_depth+1)-word
    \param trans a Translator is required for xml tree saving (if xml2 activated)
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  void select( unsigned long * count, bool decal_required,
	       const Translator & trans = Translator(), 
	       const string & xmlfile = string() )
  {    
   _nb_param = 0; 
   long decal = 0;
   if (decal_required)
     decal = _jump;
      
#ifdef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     xmlDocPtr doc=xmlNewDoc(BAD_CAST "1.0");
     xmlNodePtr root_node=xmlNewNode(NULL,BAD_CAST "treeset");
     xmlDocSetRootElement(doc,root_node);

     pmm_tree p_t = _p_f->select( count+decal );
     p_t.tree_to_matrix( _Pi );
     _nb_param += p_t.nb_leaves() * (_size-1);

     std::stringstream treenamestream;
     p_t.save( trans, root_node, treenamestream.str() );
     xmlSaveFormatFileEnc(xmlfile.c_str(),doc,"ISO-8859-1",1);
     xmlFreeDoc(doc); 
   }
   else{

     pmm_tree p_t = _p_f->select( count+decal );     
     p_t.tree_to_matrix( _Pi );
     _nb_param += p_t.nb_leaves() * (_size-1);

   }
#endif
#ifndef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     cerr<<"in PMarkov: no libxml2 detected to fill "<<xmlfile<<endl;
   } 

   pmm_tree p_t = _p_f->select( count+decal );   
   p_t.tree_to_matrix( _Pi );
   _nb_param += p_t.nb_leaves() * (_size-1);

#endif      
  }

  
  //! return the mean post likelihood over the parameters and the trees
  /*!
    \param tseq_train sequence(s) (set) for the training step
    \param initial_phase_train phase of the first element of each sequence
    \param tseq_eval sequence(s) (set) for the evaluation step
    \param initial_phase_eval phase of the first element of each sequence
  */
  template <class TSeq1, class TSeq2>
    double mean_post_log_likelihood( const TSeq1& tseq_train, const TSeq2& tseq_eval,
				     short initial_phase_train = 0, short initial_phase_eval = 0 )
  {
    tseq_train.count_occurencies();
    tseq_eval.count_occurencies();
    return this->mean_post_log_likelihood( tseq_train.get_count(), true,
					   tseq_eval.get_count(), true ); 
  }
  
  //! return the mean post likelihood over the parameters and the trees
  /*!
    \param tseq_eval sequence(s) (set) for the evaluation step
    \param initial_phase_eval phase of the first element of each sequence
  */
  template <class TSeq>
    double mean_post_log_likelihood( const TSeq& tseq_eval,
				     short initial_phase_eval = 0 )
    {     
      tseq_eval.count_occurencies();
      return this->mean_post_log_likelihood( tseq_eval.get_count(), true );
    }
  

 //! return the mean post likelihood over the parameters and the trees
  /*! 
    \param count_train count of all the coded word(base size) of size order+1, for learning step
    \param decal_required_t necessary when using a count of word from 1-word to (_max_depth+1)-word
    \param count_eval count of all the coded word(base size) of size order+1 for each phase, for evaluation step
    \param decal_required_e necessary when using a count of word from 1-word to (_max_depth+1)-word
  */
  double mean_post_log_likelihood( unsigned long * count_train, bool decal_required_t,
				   unsigned long * count_eval, bool decal_required_e )
  {              
    if (decal_required_t)        
      _p_f->extended_tree( count_train+_jump );
    else
      _p_f->extended_tree( count_train );    
   
    if (decal_required_e)     
      return _p_f->mean_post_log_likelihood( count_eval+_jump );
    else   
      return _p_f->mean_post_log_likelihood( count_eval );
  }

  //! return the mean post likelihood over the parameters and the trees
  /*! 
    \param count_eval count of all the coded word(base size) of size order+1  for evaluation step
    \param decal_required_e necessary when using a count of word from 1-word to (_max_depth+1)-word
  */
  double mean_post_log_likelihood( unsigned long * count_eval, bool decal_required_e )
  {
    if (decal_required_e) 
      return _p_f->mean_post_log_likelihood( count_eval+_jump );
    else
      return _p_f->mean_post_log_likelihood( count_eval );
  }

  //! return the mean post likelihood over the parameters and the trees
  double mean_post_log_likelihood(){
    _p_f->extended_tree();
    return  _p_f->mean_post_log_likelihood( );
  }

  //---------------------------------------
  //! compute the mean posterior likelihood over the parameters
  /*!
    \param tseq_eval sequence(s) (set) for the evaluation step
  */
  template <class TSeq>
    double post_log_likelihood( const TSeq& tseq_eval )
  {    
    if ( tseq_eval.tell_order() != _order ){
      cerr << "Order incompatibility beetween  sequence and model " << endl;
      exit(-1);
    }
    // A FAIRE attention seq > order...
    
    tseq_eval.count_occurencies();     
    
    return post_log_likelihood( tseq_eval.get_count(), true );
  }

  //! compute the mean posterior likelihood over the parameters
  /*! 
    \param count_eval count of all the coded word(base size) of size order+1 for each phase, for evaluation step
    \param decal_required_e necessary when using a count of word from 1-word to (_max_depth+1)-word
  */
  double post_log_likelihood( unsigned long * count_eval, bool decal_required_e  )
  {
    if (decal_required_e)
      return _p_f->post_log_likelihood( count_eval+_jump );
  }

  //--------------------------------------

  //! draws a model
  /*! 
    \param count count of all the coded word(base size) of size order+1, for estimation
    \param decal_required necessary when using a count of word from 1-word to (_max_depth+1)-word
    \param r random generator
    \param trans a Translator is required for xml tree saving (if xml2 activated)
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  void draw( unsigned long * count, bool decal_required,
	     gsl_rng * r,
	     const Translator & trans = Translator(), 
	     const string & xmlfile = string() )
  {
   _nb_param = 0;   
#ifdef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     xmlDocPtr doc=xmlNewDoc(BAD_CAST "1.0");
     xmlNodePtr root_node=xmlNewNode(NULL,BAD_CAST "treeset");
     xmlDocSetRootElement(doc,root_node);
     if (decal_required)
       _p_f->extended_tree( count+_jump );
     else 
       _p_f->extended_tree( count );
       
     pmm_tree p_t = _p_f->draw(r);   
     p_t.tree_to_matrix( _Pi );
     _nb_param += p_t.nb_leaves() * (_size-1);

     std::stringstream treenamestream;
     p_t.save( trans, root_node, treenamestream.str() );
     xmlSaveFormatFileEnc(xmlfile.c_str(),doc,"ISO-8859-1",1);
     xmlFreeDoc(doc); 
   }
   else{
     if (decal_required)
       _p_f->extended_tree( count+_jump );
     else
       _p_f->extended_tree( count );
       
     pmm_tree p_t = _p_f->draw(r);        
     p_t.tree_to_matrix( _Pi );
     _nb_param += p_t.nb_leaves() * (_size-1);

   }
#endif
#ifndef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     cerr<<"in PMarkov: no libxml2 detected to fill "<<xmlfile<<endl;
   } 
   if (decal_required)
       _p_f->extended_tree( count+_jump );
     else
       _p_f->extended_tree( count );
       
   pmm_tree p_t = _p_f->draw(r); 
   p_t.tree_to_matrix( _Pi );
   _nb_param += p_t.nb_leaves() * (_size-1);

#endif   

   if (_Mu != NULL)
     this->compute_stat_law( true );   
  }  
  
  //! draws a model
  /*!   
    \param r random generator  
    \param trans a Translator is required for xml tree saving (if xml2 activated)
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  void draw( gsl_rng * r,
	     const Translator & trans, 
	     const string & xmlfile = string() )
  {
   _nb_param = 0;   
#ifdef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     xmlDocPtr doc=xmlNewDoc(BAD_CAST "1.0");
     xmlNodePtr root_node=xmlNewNode(NULL,BAD_CAST "treeset");
     xmlDocSetRootElement(doc,root_node);

     pmm_tree p_t = _p_f->draw(r); 
     p_t.tree_to_matrix( _Pi );
     _nb_param += p_t.nb_leaves() * (_size-1);

     std::stringstream treenamestream;
     p_t.save( trans, root_node, treenamestream.str() );
     xmlSaveFormatFileEnc(xmlfile.c_str(),doc,"ISO-8859-1",1);
     xmlFreeDoc(doc); 
   }
   else{

     pmm_tree p_t = _p_f->draw(r);      
     p_t.tree_to_matrix( _Pi );
     _nb_param += p_t.nb_leaves() * (_size-1);

   }
#endif
#ifndef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     cerr<<"in PMarkov: no libxml2 detected to fill "<<xmlfile<<endl;
   } 
   
   pmm_tree p_t = _p_f->draw(r); 
   p_t.tree_to_matrix( _Pi );
   _nb_param += p_t.nb_leaves() * (_size-1);

#endif 

   if (_Mu != NULL)
     this->compute_stat_law( true );   
  }

  //! returns info on the number of leaves
  void info_nb_leaves() const{
    cout<<_p_f->current_pmm_tree().nb_leaves()<<" leaves"
	<<endl;
  }

  void set_penalty( double penalty ){
    _p_f->set_penalty( penalty);
  }

  void default_prior_alpha(){
    vector<double> tmp;
    tmp.assign(_size, 1.);
    _prior_alpha.assign(1, tmp);
  }

  ~PMarkov(){
    //cout<<"destr.Pmarkov"<<endl;
  }
};
#endif
