/* Partition.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
  \file
  \author Vincent Miele
  \brief Partition

   23/02/2004
*/

#ifndef PARTITION_H
#define PARTITION_H

#include <seqpp/Partition_box.h>
#include <seqpp/Translator.h>
#include <list>
#include <set>
#include <map>


/*!  
  \internal
  \brief Representation of Dealer of alphabet partitions
*/
class Partition_rep : public Partition_box< vector< short > > 
{
 private : 

  //! number of reference Partition on this representation Partition_rep
  short _countref;


  //! vector containing the list of possible elements in a partition 
  vector< vector< short > > _list_possib;
  //! vector containing vector of synonymous token as a predictor
  vector< vector<short> >  _synonymous;
  void default_synonymous(){
    for (int i=0; i<_size; i++){
      vector<short> tmp;
      tmp.push_back( i );
      _synonymous.push_back( tmp );
    }
  }

  //! index of the default motif (partition = single element with all alphabet)
  int _motifdefault;
  //! true if the default motif is added to _list_possib at the end, false if it already exists
  bool _motifdefault_skip;

  //! tools
  vector< short > _tmpvpart;
  vector< short > _tmpv;
  set< short > _tmpset;
  set< short >::const_iterator _tmpiter;
  map< vector<short> , int > _tmpvecmap;
  
  void list_possible_elements_r( short numbound,	 
		 list< short > & ,
		 list< short >::iterator & it );


  //! terminal recursion, processing the vector vec containing the group-num for each letter
  virtual void processing( short numfalse, const vector<short> & vec );

  //! initialize le mapping entre vector et numero dans _list_possib
  void init_tmpvecmap();
  //! libre l'espace
  void free_tmpvecmap(){
    _tmpvecmap.clear();
  }

 public :
  
 //! empty constructor
  Partition_rep() 
    : Partition_box< vector< short > >()
    {_countref=0;}

  //! constructor with integrated creation of the set of partitions
  /*!
    \param alphabet_size size of the alphabet
  */
  Partition_rep( short alphabet_size );
  
  //! Constructor from a file containing the partitions or the list of synonymous tokens 
  /*!
    \param alphabet a link to a Translator object required for the file reading
    \param pfile partitions file 
    <br>
    -> 1st possibility: after a "#Partition" on the first line, 
    each partition is represented as a list of tokens'subset delimited by a "|", 
    each subset being composed with tokens of the alphabet separed by space<br>
    Example:
    \code
    #Partition
    a g | c t 
    \endcode
    -> 2nd possibility:  after a "#Synonymous" on the first line,
    tokens are grouped (separed by spaces) into synonymous classes on each line<br>
    Example:
    \code
    #Synonymous
    a t
    g c
    \endcode
  */
  Partition_rep( const Translator & alphabet,
		 const string & pfile );
  
  //! destructor
  virtual ~Partition_rep()
  {};
  

  //! returns the number of reference 
  short countref() const{
    return _countref;
  }
  //! postfixed
  Partition_rep& operator ++ (int) {
    _countref++;
    return *this;
  }
  //! postfixed
  Partition_rep& operator -- (int) {
    _countref--;
    return *this;
  }



  //! initialize the list_possible_elements of possible tokens'subset in a partition and return it
  const vector< vector< short > > & list_possible_elements( );

  //! returns the synonymous list
  const vector< vector<short> > & synonymous() const{
    return _synonymous;
  }

  //! returns the number of possible tokens'subset in a partition
  long tell_nbposs() const{
    return _list_possib.size();
  }

  //! returns the size of the space to partition
  short  tell_size() const{
    return _size;
  }
  
  //! create the default motif if it not exists
  bool create_default();
  //! return the index of the default motif
  int motifdefault() const{
    return _motifdefault;
  }  
  //! true if the default motif is added to _list_possib at the end, false if it already exists
  bool motifdefault_skip() const{
    return _motifdefault_skip;
  }
  
};




/**
 * \ingroup General_Use_Markov
 */

/*!  \class Partition seqpp/Partition.h
 \brief Dealer of alphabet partitions

Let a partition of an alphabet be a set of tokens'subset, i.e. a division of the alphabet into subset.
The Partition object gives 2 possibilities:
- to compute the overall set of possible partitions (automatically generated) given the alphabet

- to compute the overall set of possible partitions (automatically generated) given a synonymous pseud-alphabet: 
by declaring synonymous tokens, it is possible to group tokens as a single predictor so that the number
 of partitions is lower.<br>
In this case, a configuration file with the top key word "#Synonymous", containing the lists of synonymous tokens, is required.<br>
Exple:
\code
#Synonymous
a t
g c
\endcode 

- to input a selected set of partitions. In this case, in a configuration file 
after a "#Partition" on the first line, 
each partition is represented as a list of tokens'subset delimited by a "|", 
each subset being composed with tokens of the alphabet separed by space<br>
Exple:
\code 
#Partition
a | g | c | t
a g | c | t
a c t | g
\endcode
*/
class Partition{
  //! representation
  Partition_rep * _rep; 

  Partition(Partition_rep& rep){
    _rep = &rep;
    (*_rep)++;
  }

 public:
  //! empty constructor
  Partition(){
    _rep = new Partition_rep();
  }
  //! constructor with integrated creation of the set of partitions
  /*!
    \param alphabet_size size of the alphabet
  */
  Partition( short alphabet_size ){
    _rep = new Partition_rep(alphabet_size);
  } 
  //! Constructor from a file containing the partitions or the list of synonymous tokens 
  /*!
    \param alphabet a link to a Translator object required for the file reading
    \param pfile partitions file 
    <br>
    -> 1st possibility: after a "#Partition" on the first line, 
    each partition is represented as a list of tokens'subset delimited by a "|", 
    each subset being composed with tokens of the alphabet separed by space<br>
    Example:
    \code
    #Partition
    a g | c t 
    \endcode
    -> 2nd possibility:  after a "#Synonymous" on the first line,
    tokens are grouped (separed by spaces) into synonymous classes on each line<br>
    Example:
    \code
    #Synonymous
    a t
    g c
    \endcode
  */  
  Partition( const Translator & alphabet,
	     const string & pfile ){
    _rep = new Partition_rep(alphabet, pfile);
  }
  //! copy constructor
  Partition(const Partition& part){
    _rep = part._rep;
    (*_rep)++;
  } 
  //! destructor
  ~Partition(){
    if (_rep->countref()>1)
      (*_rep)--;
    else
      delete _rep;
  }
  //! oeprator =
  Partition& operator=(Partition & part){
    if (this != &part){
      // delete
      if (_rep->countref()>1)
	(*_rep)--;
      else
	delete _rep;
      // new
      _rep = part._rep;
      (*_rep)++;
    }
    return (*this);
  }

  //! initialize the list_possible_elements of possible tokens'subset in a partition and return it
  const vector< vector< short > > & list_possible_elements( ){
    return _rep->list_possible_elements( );
  }
  //! returns the synonymous list
  const vector< vector<short> > & synonymous() const{
    return _rep->synonymous();
  }  
  //! return the number of partition per number of elements
  const vector< int >& nbpartition_with_nbelement() const{
    return _rep->nbpartition_with_nbelement();
  }
  //! compute and return the number of partition per number of elements, if no cutoff method previously called
  const vector< int >& compute_nbpartition_with_nbelement(){
    return _rep->compute_nbpartition_with_nbelement();
  }
  
  //! returns the number of possible tokens'subset in a partition
  long tell_nbposs() const{
    return _rep->tell_nbposs();
  }
  //! returns the size of the space to partition
  short  tell_size() const{
    return _rep->tell_size();
  }  
  //! create the default motif if it not exists
  bool create_default(){
    return _rep->create_default();
  }
  //! return the index of the default motif
  int motifdefault() const{
    return _rep->motifdefault();
  }
  //! true if the default motif is added to _list_possib at the end, false if it already exists
  bool motifdefault_skip() const{
    return _rep->motifdefault_skip();
  }
  //! cardinal of the partition
  long tell_cardinal() const{
    return _rep->tell_cardinal();
  } 

  /*! 
    \internal const iterator on the partition
  */
  class const_iterator : public Partition_rep::const_iterator {
  public:
    const_iterator( )
      : Partition_rep::const_iterator( ) {}
    const_iterator( const Partition_rep & p, long currpart )
      : Partition_rep::const_iterator( p, currpart ) {}
  };

  //! first possibility of the partition
  const_iterator begin() const{
    return const_iterator( *_rep, 0 );
  }
  //! end of list of possibilities (not a possibility)
  const_iterator end() const{
    return const_iterator( *_rep, _rep->tell_cardinal() );
  } 
};
#endif//PARTITION_H
