/* Partition_box.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*!
  \file
  \author Vincent Miele
  \brief Partition_box

  23/02/2004

  display the set of |A|-vectors containing the num of group
  for each letter. exple:
  a c g t
  1 1 1 1
  1 2 1 1
  1 2 2 1
  etc...
  1 2 3 4
  and those vector must be processed and stored in a Tpart
*/

#ifndef PARTITION_BOX_H
#define PARTITION_BOX_H

using namespace std;

#include <iostream>
#include <fstream>
#include <vector>
#include <cmath>

template <class Tpart> class Partition_box
{
 protected :
  //_________________________________________//
  //! partition computed
  vector< Tpart > _partition;
  //! the number of partition per number of elements
  vector< int > _nbpartition_with_nbelement;

  //! possibilities of partition into 2 ensbles
  bool ** _possib;
  //! nber of possible elements in a partition of 2 ensbles
  int _nbposs;
  //! power of 2
  int * _pow;

  //! size of the ensemble
  short _size;

  //! cardinal of the whole partitions'set
  long _cardinal;

  //! current partition
  long _currpart;

  //_________________________________________//



  //! compute _possib
  void init_possib(){
    int i,j;
    short si;
    _pow = new int[_size+1];
    _pow[0] = 1;
    for (si=0; si<_size; si++){
      _pow[si+1] = 2*_pow[si];
    }
    _nbposs = _pow[_size-1];
    
    _possib = new bool*[_nbposs];
    for (i=0; i<_nbposs; i++)
      _possib[i] = new bool[_size]; 
    
    int pow1 = 1, pow2=_nbposs/2;
    for (i=0; i<_nbposs; i++)
      _possib[i][0] = false;
    for (si=1; si<_size; si++){
      for (i=0; i<pow1; i++)
	for (j=0; j<pow2; j++){
	  _possib[(2*i+1)*pow2+j][si] = false;
	  _possib[2*i*pow2+j][si] = true;
	}
      pow1 *= 2;
      pow2 /= 2;
    }
    /*
      for (i=0; i<_nbposs; i++){
      for (si=0; si<_size; si++)
      cout<<_possib[i][si]<<" ";
      cout<<endl;
      }
    */
  }

  
  //! delete _possib
  void delete_possib(){
    int i;
    for (i=0; i<_nbposs; i++){
      delete [] _possib[i];
    }
    delete [] _possib;
  }  

  long approx_cardinal(){
    int maxIter = 30;
    double  epsilon = 0.0001, test = 1., kpuiss, sum = 0, sumold;
    double i = 1, kfact = 1;
    while  ((i<maxIter)&&(abs(test)>epsilon))
      {
	sumold = sum;
	kpuiss = pow(double(i), double(_size));
	kfact = kfact*i;
	sum = sum+kpuiss/kfact;
	test = sumold-sum;
	i++;
      }
    return  ( long(sum*(exp(-1.))) +1);
  }
  
  //! recursive tool
  void cutoff_r( vector<short> & vec, 
		 short nbmaxfalse, short numfalse,
		 short notaffected ){
    int i, j;
    short snif, newnbfalse=0;
  
    if (notaffected==0){
      // processing of the successfull possibility
      //------------------------------------------
      this->processing( numfalse, vec );

      /*
	cout<<"-----> Proposition : ";
	for ( j=0; j<_size; j++ )
	cout<<vec[j]<<" ";
	cout<<endl;
      */
      // end processing of the successfull possibility
      //----------------------------------------------
      _cardinal++;
    }
    else{
      // jump to avoid synonymous possibilities
      int jump = _pow[_size-notaffected];
      // all the possibilities
      for ( i=0; i<_nbposs; i+=jump ){
	/*
	  cout<<"Possibility : ";
	  for ( j=0; j<notaffected; j++ )
	  cout<<_possib[i][j]<<" ";
	  cout<<endl;
	*/

	snif = 0;
	newnbfalse = 0;
	// "copy" of the possibility in the vector
	for ( j=0; j<_size; j++ ){	
	  // if not affected
	  if ( vec[j]==0 ){
	    // FOR EACH FALSE, you put a NUMFALSE
	    if ( _possib[i][snif++] == false ){
	      vec[j] = numfalse;
	      newnbfalse++;
	    }
	  }
	}
  
	cutoff_r( vec, 
		  newnbfalse,  numfalse+1,
		  notaffected-newnbfalse );
 
	for ( j=0; j<_size; j++ )
	  if ( vec[j] == numfalse )
	    vec[j] = 0;
      }
    }
  }

  //! terminal recursion, processing the vector vec containing the group-num for each letter
  virtual void processing( short numfalse,
			   const vector<short> & vec ) = 0;

 public :   

  //! default constructor
  Partition_box( ){
    _size = 0;
    _currpart = 0;
    _cardinal = 0;
    _pow = NULL;
    _nbposs = 0;
    _possib = NULL;
  }

  //! constructor
  Partition_box( short size_ensemble ){
    _size = size_ensemble;
    _currpart = 0;
    if (size_ensemble<14){
      long card = approx_cardinal();  
      /* 
	 cout<<"max: "<<_partition.max_size()<<endl;      
	 cout<<"card: "<<card<<endl;   
	 if (card > _partition.max_size()){
	 cerr<<"Partition_box: No space left -> alphabet too wide -> too many possible partitions"
	 <<endl;
	 exit(1);
	 }
      */
      _partition.reserve( card );
    } 
    _cardinal = 0;
    _pow = NULL; 
    _nbpartition_with_nbelement.assign(_size,0);
  }

  //! copy constructor
  Partition_box( const Partition_box & p ){
    _size = p._size;
    _currpart = p._currpart;
    _partition.reserve = p._partition;    
    _cardinal = p._cardinal;
    _pow = NULL;
  }
  
  //! destructor
  virtual ~Partition_box(){
    if (_pow)
      delete [] _pow;
  };

  //! compute the partition
  void cutoff(){
    vector<short> v(_size, 0);
    this->init_possib();
    _cardinal = 0;
    cutoff_r(v, _size, 1, _size);
    this->delete_possib();
  }

  //! cardinal of the partition
  long tell_cardinal() const{
    return _cardinal;
  }
  
  //! size of the ensemble
  long tell_size() const{
    return _size;
  }

  //! return ith elt
  const Tpart & element( int i) const{
    return _partition[i];
  }

  //! return the number of partition per number of elements
  const vector< int >& nbpartition_with_nbelement() const{
    return _nbpartition_with_nbelement;
  } 

  //! compute and return the number of partition per number of elements, if no cutoff method previously called
  const vector< int >& compute_nbpartition_with_nbelement(){
    int i, s= _partition.size();
    _nbpartition_with_nbelement.clear();
    _nbpartition_with_nbelement.assign(_size,0);
    for (i=0; i<s; i++)
      _nbpartition_with_nbelement[ _partition[i].size()-1 ]++;
    return _nbpartition_with_nbelement;
  }

  //_________________________________________// 
  class const_iterator{
  private :
    const Partition_box<Tpart> * _part_box; 
    long _currpart;
  protected : 
    friend class Partition_box;
    const_iterator( const Partition_box<Tpart> & p, long currpart ){
      _part_box = &p;
      _currpart = currpart;
      if (p._cardinal == 0)
	cerr<<" cutoff required !"<<endl;
    }
  public :
    const_iterator( ){
      _currpart = -1;
      _part_box = NULL;
    } 
    //! prfixed. return false if the next is the end()
    bool operator ++ () {
      // check if the next is the end()
      if ( _currpart<_part_box->_cardinal-1 ){
	_currpart++;
	return true;
      }
      else{
	_currpart++;
	return false;
      } 
    }
    //! postfixed. return false if the next is the end()
    bool operator ++ (int) {
      // check if the next is the end()
      if ( _currpart<_part_box->_cardinal-1 ){
	_currpart++;
	return true;
      }
      else{
	_currpart++;
	return false;
      }
    }
    //! operator =!
    bool operator != ( const const_iterator & p ) const{
      return ( (p._part_box !=_part_box) 
	       || (p._currpart !=_currpart) );
    }
    //! operator ==
    bool operator == ( const const_iterator & p ) const{
      return ( (p._part_box ==_part_box) 
	       && (p._currpart ==_currpart) );
    }
    //! operator *
    const Tpart& operator * () const{
      return _part_box->_partition[_currpart];
    }

    const_iterator operator = (  const const_iterator & p ){
      if (  p != *this ){
	_part_box =  p._part_box;
	_currpart = p._currpart;
      }
     return *this;
    }
  };

  //! first possibility of the partition
  const_iterator begin() const{
    return const_iterator( *this, 0 );
  }
  //! end of list of possibilities (not a possibility)
  const_iterator end() const{
    return const_iterator( *this, _cardinal );
  }  
  //_________________________________________//
};


#endif //PARTITION_BOX_H
