/* seqpp/PhasedMTDMarkov.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
 \file
  \author Vincent Miele
  \brief  Phased Mixture Transition Distribution  Markov model

   01/2004
*/

#ifndef SEQPP_PHASEDMTDMARKOV_H
#define SEQPP_PHASEDMTDMARKOV_H

#include <seqpp/PhasedMarkov.h>
#include <seqpp/mtd_core.h>

/**
 * \ingroup General_Use_Markov
 */

/*!  \class PhasedMTDMarkov seqpp/PhasedMTDMarkov.h
  \brief   Phased Mixture Transition Distribution Markov modelling, estimation and simulation.

  PhasedMTDMarkov is a PhasedMarkov object with a different estimation step.
  This object performs the estimation with the Mixture Transition Distribution algorithm 
  and then transfoms, once per phase, the  MTD model in a markovian matrix.
*/
class PhasedMTDMarkov : public PhasedMarkov
{
 protected : 
  mtd_core * _mtdcore;

 public:
  //! Constructor 1 from a SequenceSet or a Sequence
  /*!
    \param  tseq a set of sequences or a sequence for estimation
    \param mkv_order markovian order on the markovian matrix of the MTD
    \param phase phase short 
    \param initial_phase phase of the first element of each sequence
    \param nbseed number of seeds for the EM algorithm
    \param nbiter_max maximum iterations number of the EM algorithm
    \param eps value of the epsilon of the EM algorithm
    \param log true to log the successive likelihood values
  */
  template< class TSeq >
  PhasedMTDMarkov( const TSeq & tseq, short mkv_order,
		   short phase, short initial_phase = 0,
		   short nbseed = NBSEED, int nbiter_max = NBITERMAX, double eps = EPS,
		   bool log = false, bool identical_pi = false )
    :  PhasedMarkov(  tseq.tell_alphabet_size(), tseq.tell_order(), phase )
  { 
    tseq.count_p_occurencies( phase, initial_phase ); 
    estimate( tseq.get_p_count(), true, mkv_order, phase, initial_phase, 
	      nbseed, nbiter_max, eps, log, identical_pi );
  }
 
  //! Constructor 2 from a coded-word count
  /*!
    \param count count of all the coded word(base size) of size order+1 for each phase, for estimation
    \param size alphabet size
    \param mtd_order markovian order for the resulting model
    \param mkv_order markovian order on the markovian matrices used in the MTD
    \param phase phase 
    \param initial_phase phase of the first element of each sequence
    \param nbseed number of seeds for the EM algorithm
    \param nbiter_max maximum iterations number of the EM algorithm
    \param eps value of the epsilon of the EM algorithm
    \param log true to log the successive likelihood values
  */
  PhasedMTDMarkov( unsigned long * * count,
		   short size, 
		   short mtd_order, short mkv_order,
		   short phase, short initial_phase = 0,
		   short nbseed = NBSEED, int nbiter_max = NBITERMAX, double eps = EPS,
		   bool log = false, bool identical_pi = false )
    :  PhasedMarkov( size, mtd_order, phase )
  {       
    estimate( count, false, mkv_order, phase, initial_phase, 
	      nbseed, nbiter_max, eps, log, identical_pi );
  }

  //! performs the estimation [used by each constructor] 
  void estimate( unsigned long * * count, bool decal_required,
		 short mkv_order,
		 short phase, short initial_phase,
		 short nbseed, int nbiter_max, double eps,
		 bool log, bool identical_pi )
 {  
   _mtdcore = new mtd_core( _size, _order, mkv_order, identical_pi );
   
   _nb_param = 0; 

   for (int i=0; i<phase; i++){    
     _nb_param += _mtdcore->nb_params();
     _mtdcore->estimate( count[i], decal_required,
			 nbseed, nbiter_max, eps, log );
     _mtdcore->mtd_to_matrix( _Pis[i] );
   }
   delete _mtdcore;
 }
 
//! Destructor
 ~PhasedMTDMarkov(){} 
};
#endif
