/* seqpp/PhasedMarkov.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
 \file
  \author David Robelin - Vincent Miele
  \brief  Phased Markov model

   01/2004
*/

#ifndef SEQPP_PHASEDMARKOV_H
#define SEQPP_PHASEDMARKOV_H

using namespace std;
#include <seqpp/SequenceSet.h>
#include <seqpp/arnoldi.h>
#include <vector>
#include <string>

#include <gsl/gsl_sf_gamma.h>

/**
 *  \internal
 *  \defgroup General_Use_Markov Library Design: Classes dedicated to Markov-related modelling/estimation/simulation/statistics
*/


/**
 * \ingroup General_Use_Markov
 */

/*!  \class PhasedMarkov seqpp/PhasedMarkov.h
\brief Phased Markov modelling, estimation and simulation.

 This is generalization of a Markov chain, using different matrices in function of the considered position in the sequence. The phase is variable.<BR>
For example, if we consider 3 phases, and we note respectively Pi1, Pi2 and Pi3, the three 
transition matrices, the Markov sequences will be generated by the indices 123123123123...
In a DNA modelisation (genomic field), this is useful to take into account the fact that a 
coding region is read by 3 bases-blocks. The order of the Markov
Model, i.e. the number of previous states necessary to determine the distribution of the
current state, is variable. It is assume here that this order is the same in all the phases.<BR>

Methods are implemented for Markovian transition matrix estimation, stationary distribution calculus, word probabilities, total variation distance between two Markovian matrices, and further. The efficiency of eigenproblems computation is ensured by the use of the implicitly restarted Arnoldi algorithm.<BR> Simulations are also possible.<BR>
*/

class PhasedMarkov
{

public :

  //! Constructor 1 : read a configuration file
  /*
    \param markov_file description of an input model in a file (generated by print method) 
    \param calc_rank calculus of the convergence rank if true
    Exple of file:
    \code
    # 4 <- Order of the phased Markov chain<br>
    # 3 <- Phase<br>
    # 4 <- Alphabet size<br>
    # Phase n: 0<br>
    # 10 steps <- Convergence to the stationnary distribution (!!Optionnal!!)<br>
    # Transition matrix:<br>
    ...<br>
    # Stationnary Probability: (!!Optionnal!!)<<br>
    ...
    \endcode
  */
  PhasedMarkov(  const string & markov_file,
		 bool calc_rank = false  );
 
  //! Constructor 2 : Estimate the transition matrices on the sequences of seqset
  /*!
    \param seqset set of sequences for estimation
    \param phase selected phase
    \param initial_phase phase of the first element of each sequence 
    \param calc_rank calculus of the convergence rank if true
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element, and for each phase (separated by a "#Phase i")
  */
  PhasedMarkov( const SequenceSet & seqset,
		short phase, short initial_phase = 0,
		bool calc_rank = false,
		const string& prior_alpha_file = string() );
  
  //! Constructor 3 : Estimate the transition matrices on the sequence seq
  /*!
    \param seq sequence for estimation
    \param phase selected phase
    \param initial_phase phase of the first element of each sequence 
    \param calc_rank calculus of the convergence rank if true
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element, and for each phase (separated by a "#Phase i")
  */
  PhasedMarkov( const Sequence & seq,
		short phase, short initial_phase = 0,
		bool calc_rank = false,
		const string& prior_alpha_file = string() );

  //! Constructor 4 : Copy constructor
  PhasedMarkov( const PhasedMarkov & phm );

  //! Constructor 5 : Default constructor
  PhasedMarkov();

  //! Constructor 6 : Minimal Constructor
  /*!
    Initialises the constants of the model but not the matrices nor the stat laws
    \param size alphabet size
    \param order markovian order
    \param phase selected phase
    \param alloc true for matrices memory allocation
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element, and for each phase (separated by a "#Phase i")
  */
  PhasedMarkov( short size, short order, short phase, bool alloc=true,
		const string& prior_alpha_file = string() );

  //! Constructor 7 : Creation of a "mixed" Markov chain M = p*M1 + (1-p)*M2 */
  /*!
    \param M1 first Markov chain object
    \param M2 second Markov chain object
    \param p weight of M1 in the resulting M(with 0<=p<=1)
  */
  PhasedMarkov(const PhasedMarkov &M1, const PhasedMarkov &M2, const float p);

  //!Constructor 8 : Estimation of the transition matrix based on the sequences of seqset given in Indseq
  /*!
    \param seqset set of sequences for estimation
    \param Indseq index of selected sequences
    \param phase selected phase
    \param initial_phase phase of the first element of each sequence 
    \param calc_rank calculus of the convergence rank if true
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element, and for each phase (separated by a "#Phase i")
  */
  PhasedMarkov(const SequenceSet & seqset, 
	       const vector<int> &Indseq, 
	       short phase, short initial_phase = 0,
	       bool calc_rank = false,
	       const string& prior_alpha_file = string() ); 

  //! Constructor 9 : random markov matrices 
  /*!
    \param r gsl random generator
    \param size alphabet size
    \param order markovian order
    \param phase selected phase
    \param calc_rank calculus of the convergence rank if true

    GSL use exple:<br> 
    \code
    const gsl_rng_type * T;
    // Choice a default generator and seed
    // from environment variables
    gsl_rng_env_setup();
    // New created instance of the generator
    T = gsl_rng_default;
    gsl_rng * r = gsl_rng_alloc (T);
    // Initialize/Seeds the random number generator
    gsl_rng_set( r, (long)getpid() );
    ...
    ...
    gsl_rng_free( r );
    \endcode    
  */
  PhasedMarkov( const gsl_rng * r,
		short size, short order, 
		short phase, 
		bool calc_rank = false );

  //! Constructor 10  Estimate the transition matrices on a word-count
  /*!
    \param count count of all the coded word(base size) of size order+1 for each phase,for estimation
    \param size alphabet size
    \param order markovian order
    \param phase selected phase
    \param initial_phase phase of the first element of each sequence 
    \param calc_rank calculus of the convergence rank if true
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element, and for each phase (separated by a "#Phase i")
  */
  PhasedMarkov( unsigned long * * count,
		short size, short order,
		short phase, short initial_phase = 0,
		bool calc_rank = false,
		const string& prior_alpha_file = string() );

  //! Destructor
  virtual ~PhasedMarkov();
  
  //---------------------------------------
  //! Estimate the transition matrices on the sequence/sequenceset tseq
  /*!
    \param tseq sequence/sequenceset for estimation
    \param phase selected phase
    \param initial_phase phase of the first element of each sequence 
    \param beg begin position in sequence(s) if subsequences 
    \param end end position in sequence(s) if subsequences  
    \param calc_rank calculus of the convergence rank if true  
    \param count_again false if the word-count already performed before the use of this method 
  */
  template <class TSeq>
    void estimate( const TSeq & tseq,
		   short phase, short initial_phase, 
		   unsigned long beg, unsigned long end,
		   bool calc_rank = false, bool count_again = true )
    {
      if (count_again)
	tseq.clear_count();  
      tseq.count_p_occurencies(_phase,initial_phase,beg,end);
      
      estimate( tseq.get_p_count(), true, calc_rank );
    } 


  //! Estimate the transition matrices from a file containing the count
  /*!
    \param count_file file containing the count in the adapted format
    \param calc_rank calculus of the convergence rank if true
  */
  void estimate( const string& count_file, bool calc_rank= false )
  {
    unsigned long ** count = new unsigned long*[_phase];
    for (short p=0; p<_phase; p++ )
      count[p] = new unsigned long[_nPi];    

    if (!load_count( count_file, count )){
      cerr<<"Bad count file"<<endl;
      exit(1);
    }
      
    estimate( count, false, calc_rank );

    for (short p=0; p<_phase; p++ )
      delete[] count[p];
    delete [] count;    
  }

  //! Estimate the transition matrices from a word count
  /*! 
    \param count count of all the coded word(base size) of size order+1 for each phase, for estimation 
    \param decal_required necessary when using a count of word from 1-word to (_order+1)-word
    \param calc_rank calculus of the convergence rank if true 
  */
  void estimate( unsigned long * * count, bool decal_required, bool calc_rank= false );


  //---------------------------------------
  //! access to the markov matrix(ces)
  const double ** markov_matrices( ) const{
    return const_cast< const double** >(_Pis);
  }
  //! access to the numphase-th markov matrix
  const double * markov_matrix( short numphase ) const{
    if ( numphase<_phase )
      return _Pis[numphase];
    else 
      return NULL;
  }
  
  //! draw at random the markov matrices 
  /*!
    \param r gsl random generator

    GSL use exple:<br> 
    \code
    const gsl_rng_type * T;
    // Choice a default generator and seed
    // from environment variables
    gsl_rng_env_setup();
    // New created instance of the generator
    T = gsl_rng_default;
    gsl_rng * r = gsl_rng_alloc (T);
    // Initialize/Seeds the random number generator
    gsl_rng_set( r, (long)getpid() );
    ...
    ...
    gsl_rng_free( r );
    \endcode    
  */
  void draw_markov_matrices(const  gsl_rng * r);


  //! allocate the memory for markov matrices
  inline virtual void new_markov_matrices();
  //! free the memory allocated for markov matrices
  inline virtual void free_markov_matrices();

  //! Total variation distance between *this and M
  double total_variation( const PhasedMarkov & M );


  //---------------------------------------
  //! Compute the stationnary laws
  /*
    \param force "true" to force the calculation, even if the laws already exist. Default => "false". 
  */ 
  void compute_stat_laws( bool force = false ); 
  //! access to the stationnary distrib in phase numphase
  const double * stat_law( short numphase = 0 ) const{
    if ( numphase<_phase )
      return _Mus[numphase];
    else 
      return NULL;
  }
  //! free the memory allocated for stationnary laws
  inline void free_stat_laws();
  
  //! Get the empirical relative frequency of the first order+1 letters on the set of sequences "seqset"
  /*
    \param MuInit initial law, must have allocated
    \seqset set of sequences
  */
  void compute_init_law( double * MuInit, const SequenceSet & seqset ) const;


  //---------------------------------------
  //! Computes the rank of convergence of the Markov Chain
  virtual int compute_rank();   
  //! return the number of effective parameters
  virtual long nb_parameters() const{
    return  _nb_param;
  }


  //---------------------------------------
  //! link to a Translator object to use proba methods with strings 
  void link_to_translator( const Translator & trans ){
    _trans = &trans;
  }
  //! Stationnary proba of a word(size greater than _order) conditionnaly of its first letters (!use link_to_translator before!)
  /*!
    \param word word as a string -a Translator object is required to process string to int-
    \param coder required Coder object, from a PrimaryCount object for exple
    \param numphase  phase of the last letter of word
  */
  double proba_c( const string & word, 
		  Coder & coder, short numphase=0 ) const;
  //! Stationnary proba of a word 
  /*!
    \param word word as a string -a Translator object is required to process string to int-
    \param coder required Coder object, from a PrimaryCount object for exple
    \param numphase  phase of the last letter of word
  */
  double proba( const string & word, 
		Coder & coder, short numphase=0 ) const;


  //! Stationnary proba of a word(size greater than _order) conditionnaly of its first letters
  /*!
    \param word word as a vector of short
    \param coder required Coder object, from a PrimaryCount object for exple
    \param numphase  phase of the last letter of word
  */
  double proba_c( const vector<short> & word, 
		  Coder & coder, short numphase=0 ) const;
  //! Stationnary proba of a word 
  /*!
    \param word word as a vector of short
    \param coder required Coder object, from a PrimaryCount object for exple
    \param numphase  phase of the last letter of word
  */
  double proba( const vector<short> & word, 
		Coder & coder, short numphase=0 ) const;


  //! Stationnary proba of a word(size greater than _order) conditionnaly of its first letters
  /*!
    \param word word as a Sequence-coded-like integer (see Sequence)
    \param lw length of the word. Default => order+1
    \param jump  Sequence-coded-like jump (see Sequence). Default => jump[order]
    \param numphase  phase of the last letter of word
  */
  double proba_c( long word, int lw=-1, long jump=-1, short numphase=0 ) const;
  //! Stationnary proba of a word 
  /*!
    \param word word as a Sequence-coded-like integer (see Sequence)
    \param lw length of the word. Default => order+1
    \param jump  Sequence-coded-like jump (see Sequence). Default => jump[order]
    \param numphase  phase of the last letter of word
  */
  double proba( long word, int lw=-1, long jump=-1, short numphase=0 ) const;

  
  //! Stationnary proba of the word seq[tbeg...tend](size greater than _order) conditionnaly of its first letters with seq[tend] in phase numphase
  /*!
    \param seq Sequence-like coded sequence(see Sequence). The Markov-order for the code must be the same than _order
    \param tbeg begin position of the word
    \param tend end position of the word
    \param numphase  phase of the last letter of word
  */
  double proba_c(const long * seq, long tbeg, long tend, short numphase=0) const;

  //! Stationnary proba of the word seq[tbeg...tend] with seq[tend] in phase numphase
  /*!
    \param seq Sequence-like coded sequence(see Sequence). The Markov-order for the code must be the same than _order
    \param tbeg begin position of the word
    \param tend end position of the word
    \param numphase  phase of the last letter of word
  */
  double proba(const long * seq, long tbeg, long tend, short numphase=0) const;
 
 
  //---------------------------------------
  //! loglikelihood of a set of sequence
  /*!
    \param seqset set of sequence
    \param initial_phase phase of the first element of each sequence
    \param numphase likelihood in only the numphase phase, -1 if sum of the phases
  */
  double log_likelihood( const SequenceSet & seqset,
			 short initial_phase = 0, short numphase=-1 ) const; 

  //! Calculation of the logarithm of the ratio of the probability of observing "seq" under "this" distribution and "M"
  /*!
    \param seqset set of sequences
    \param M alternative Markov chain
    \param initial_phase1 phase of the first element of each sequence considered in model *this
    \param initial_phase2 phase of the first element of each sequence considered in model M
    <br>REMARKS : NO verification is done on the compatibility of the
    Markov chains<br>
    CAREFUL: this calculus is performed by scanning the sequence to avoid pbm of too low likelihood
  */
  double log_ratio_likelihood( const SequenceSet & seqset,
			       const PhasedMarkov &M,
			       short initial_phase1=0,				     
			       short initial_phase2=0 ) const ; 


   //! loglikelihood of a sequence
  /*!
    \param seq sequence
    \param initial_phase phase of the first element of each sequence
    \param numphase likelihood in only the numphase phase, -1 if sum of the phases
  */
  double log_likelihood( const Sequence & seq,
			 short initial_phase = 0, short numphase=-1 ) const; 

  //! Calculation of the logarithm of the ratio of the probability of observing "seq" under "this" distribution and "M"
  /*!
    \param seq sequence
    \param M alternative Markov chain
    \param initial_phase1 phase of the first element of each sequence considered in model *this
    \param initial_phase2 phase of the first element of each sequence considered in model M
    <br>REMARKS : NO verification is done on the compatibility of the
    Markov chains<br>
    CAREFUL: this calculus is performed by scanning the sequence to avoid pbm of too low likelihood
  */
  double log_ratio_likelihood( const Sequence & seq,
			       const PhasedMarkov &M,
			       short initial_phase1=0,				     
			       short initial_phase2=0 ) const ;
  
  //! BIC of sequences (BIC = -2*loglikelihood + nbparam*log(length))
  /*!
    \param tseq a sequence or a set of sequences
    \param initial_phase phase of the first element of each sequence 
  */
  template <class TSeq>
    double BIC( const TSeq & tseq, short initial_phase=0 ) const{   
    //BIC = -2*loglikelihood + nbparam*log(length)
    return( -2*log_likelihood( tseq, initial_phase ) + _nb_param*log( (double)tseq.tell_length() ) );
  }

 
  //! AIC of a set of sequences (AIC = -2*loglikelihood + 2*nbparam)
  /*!
    \param tseq a sequence or a set of sequences
    \param initial_phase phase of the first element of each sequence 
  */
  template <class TSeq>
    double AIC( const TSeq & tseq, short initial_phase=0 ) const{
    //AIC = -2*loglikelihood + 2*nbparam
    return( -2*log_likelihood( tseq, initial_phase )  + 2*_nb_param );
  }


  //---------------------------------------
  //! compute the mean posterior likelihood over the parameters
  /*!
    \param tseq_train [set of] sequence for the training step
    \param tseq_eval [set of] sequence for the evaluation step
    \param force "true" to force the re-calculation on tseq_train. Default => "false"
    \param initial_phase_train phase of the first element of each sequence for the training step
    \param initial_phase_eval phase of the first element of each sequence for the evaluation step
  */
   template <class TSeq1, class TSeq2>
   double post_log_likelihood( const TSeq1& tseq_train, const TSeq2& tseq_eval,
			       bool force=false,
			       short initial_phase_train = 0, short initial_phase_eval = 0 )
   {
    double LogLik_te = 0., LogLik_t = 0.,sum1=0., sum2=0.;
    short p, u;
    long j, word;

    if (( tseq_train.tell_order() != _order )||( tseq_eval.tell_order() != _order )){
      cerr << "Order incompatibility beetween  sequence and model " <<endl;
      exit(-1);
    }

    if (_prior_alpha.size() == 0){
       cerr << "Prior input required for post_log_likelihood." <<endl;
      exit(-1);
    }
      
    // A FAIRE attention seq > order...

    tseq_train.count_p_occurencies( _phase, initial_phase_train );
    tseq_eval.count_p_occurencies(_phase, initial_phase_eval);
       
    unsigned long * countp_t, * countp_e;
    long jump = tseq_train.tell_jump();

   
    vector<double> lng1; 
    lng1.reserve(_size);
    for (p=0; p<_phase; p++){
      
      double sum = 0;
      for (u=0; u<_size; u++){
	lng1[u] = gsl_sf_lngamma(_prior_alpha[p][u]);
	sum += _prior_alpha[p][u];
      }
      double lng2 = gsl_sf_lngamma(sum);
      
      countp_t = (tseq_train.get_p_count())[p] + jump;
      countp_e = (tseq_eval.get_p_count())[p] + jump;

	for(j = 0; j < _nMu; j++){	
	  sum1=0., sum2=0.;
	  word = j*_size;
	  for (u=0; u<_size; u++){
	    sum1 = countp_t[word+u] + countp_e[word+u] + _prior_alpha[p][u];
	    LogLik_te += gsl_sf_lngamma(sum1) - lng1[u];
	    sum2 += sum1;
	  }
	  LogLik_te += lng2 - gsl_sf_lngamma(sum2);
	} 
    }
    
    if ((_postloglike>0)||(force==true)){
      for (p=0; p<_phase; p++){

	double sum = 0;
	for (u=0; u<_size; u++){
	  lng1[u] = gsl_sf_lngamma(_prior_alpha[p][u]);
	  sum += _prior_alpha[p][u];
	}
	double lng2 = gsl_sf_lngamma(sum);
	
	countp_t = (tseq_train.get_p_count())[p] + jump;
	for(j = 0; j < _nMu; j++){	
	  sum1=0., sum2=0.;
	  word = j*_size;
	  for (u=0; u<_size; u++){
	    sum1 = countp_t[word+u] + _prior_alpha[p][u];
	    LogLik_t += gsl_sf_lngamma(sum1) - lng1[u];
	    sum2 += sum1;
	  }
	  LogLik_t += lng2 - gsl_sf_lngamma(sum2);
	}	
      }
      _postloglike = LogLik_t;
    }
    
    return(LogLik_te - _postloglike);
   }

  
   
   //---------------------------------------
   //! Print a summary of the object
   /*!
     The estimation results can be saved in such a representation:
    \code
    # 1 <- Order of the phased Markov chain
    # 2 <- Phase
    # 4 <- Alphabet size
    # 19 steps <- Convergence to the stationnary distribution
    # Phase n0
    # Transition matrix:
    0.3945322543    0.1652811616    0.1535033485    0.2866832356
    etc...........
    # Stationnary Probability:
    0.3127105148    0.2114684268    0.1783495332    0.2974715251
    # Phase n1
    # Transition matrix:
    0.3923961961    0.163516403     0.1521005152    0.2919868858
    etc................
    # Stationnary Probability:
    0.3135417652    0.2089660861    0.1771006767    0.300391472
    \endcode
  */
   inline void print( const string &FileOut );
  
   //! Print a summary of the object 
  void print(ofstream &Out) const;

  //---------------------------------------
   //! Returns the alphabet size
  int tell_size() const { return(_size);};
  //! Returns the convergence rank
  int tell_rank() const { return(_rank);};
  //! Returns the order
  int tell_order() const { return(_order);};
  //! Returns the phase
  int tell_phase() const { return(_phase); }

  //---------------------------------------
  //! size of the stat law vector
  int nMu() const { return(_nMu); }
  //! size of the matrix
  int nPi() const { return(_nPi); }

  //! Access to Markov matrix Pi
  /*!
    \param index index of the word
    \param p selected phase 
  */
  double Pi(int index, int p=0) const { 
    return(_Pis[p][index]); 
  }
  //! () operator for Markov matrix Pi elements
  /*!
    \param index index of the word
    \param p selected phase 
  */
  double & operator() (int index, int p=0){ 
    return(_Pis[p][index]); 
  }

  //! Access to stationnary vector Mu elements
  /*!
    \param index index of the word
    \param p selected phase 
  */
  double Mu(int index, int p=0) const { 
    return(_Mus[p][index]); 
  }  
  //!_Pis != NULL ?
  bool isPis() const { return(_Pis != NULL);};
  //!_Mus != NULL ?
  bool isMus()  const { return(_Mus != NULL);};


  //! Give the phase following p
  inline short nextPhase(short p) const;
  //! Give the phase preceding p
  inline short prevPhase(short p) const;
  //--------------------------------------- 

  //! Verify stochasticity of the _Pis[] and eventually rescale it.
  bool Stochasticity();

  //! fill a count from a file
  /*!
    \param src_file file containing the count, phase per phase, in alphabetical order
    \param dest_count destination count (must be _phase X _nPi)
  */
  void file_to_count( const string& src_file, unsigned long ** dest_count);
   
 protected :
  
  //!Phase of the model
  short _phase;

  //! Pointer to "Matrices" (in vector format) of transition probabilities for each phase
  double **_Pis;
  //! Container of "Matrices" (in vector format) of transition probabilities for each phase
  double **_containers; 

  //!Vector of stationnary probabilities for each phase
  double **_Mus;   
  
  //!Size of the alphabet
  short _size; 
  
  //!Order of the model (the same at each phase)
  short _order;
  
  //!Dim of Pi :_size^(_order+1)
  long _nPi; 

  //!Dim of Mu :_size^_order 
  long _nMu; 
 
  //! number of effective parameters
  long _nb_param;

  //!How many steps to converge to Mu ?
  int _rank;

  //!jump to the codes of _order+1 letters when Sequence-like code
  long _jump;

  //!(Optimization) For each phase, give the next phase
  short *_nextPhase;

  //!(Optimization) For each phase, give the previous phase
  short *_prevPhase;

  //! link to a translator object for the use of proba methods
  const Translator * _trans;

  //! current posterior likelihood from a training [set of] sequence
  double _postloglike;

  //! prior on the counts, one value per alphabet element, and for each phase
  vector< vector<double> > _prior_alpha;


  inline void FreeNextPhase();
  inline void FreePrevPhase();

  //!_nextPhase != NULL ?
  bool isNextPhase()  const { return(_nextPhase != NULL);};
  //!_prevPhase != NULL ?
  bool isPrevPhase()  const { return(_prevPhase != NULL);}; 

  bool load_count( const string & count_file, unsigned long** data );

  bool load_prior_alpha( const string & prior_alpha_file );
};


inline short PhasedMarkov::nextPhase(short p) const
{
  return (_nextPhase[p]);
}

inline short PhasedMarkov::prevPhase(short p) const
{
  return (_prevPhase[p]);
}

inline void PhasedMarkov::new_markov_matrices()
{   
  if (!_Pis){
    _containers = new double*[_phase];
    _Pis = new double*[_phase];
    for (short p=0; p<_phase; p++){
      _containers[p] = new double[_nPi];
      _Pis[p] = _containers[p];
    }
  }
}

inline void PhasedMarkov::free_markov_matrices()
{
  if (_Pis){
    for (int p=0;p<_phase;p++)
      delete []_containers[p];
    delete []_containers;
    
    delete []_Pis;
  }
  _Pis = NULL;
  _containers = NULL;
}

inline void PhasedMarkov::free_stat_laws()
{
  if (_Mus)
    {
      for (int p=0;p<_phase;p++)
	delete []_Mus[p];
      delete []_Mus;
    }
  _Mus = NULL;
}

inline void PhasedMarkov::FreeNextPhase()
{
  if (isNextPhase())
    delete [] _nextPhase;
  _nextPhase = NULL;
}

inline void PhasedMarkov::FreePrevPhase()
{
  if (isPrevPhase())
    delete [] _prevPhase;
  _prevPhase = NULL;
}


inline void PhasedMarkov::print(const string & FileOut )
{
  ofstream Out(FileOut.c_str());
  print(Out);
}

#endif/*SEQPP_PHASEDMARKOV_H*/

