/* seqpp/PhasedPMarkov.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
 \file
  \author Vincent Miele
  \brief  Phased Parcimonious Markov model

   01/2004
*/

#ifndef SEQPP_PHASEDPMARKOV_H
#define SEQPP_PHASEDPMARKOV_H

#include <seqpp/PhasedMarkov.h>

#include <seqpp/pmm_forest.h>

/**
 * \ingroup General_Use_Markov
 */

/*!  \class PhasedPMarkov seqpp/PhasedPMarkov.h
  \brief   Phased Parcimonious Markov modelling, estimation and simulation.

  PhasedPMarkov is a PhasedMarkov object with a different estimation step.
  This object performs the estimation with the Parcimonious Markov algorithm 
  and then transfoms, once per phase, the parcimonious context tree in a markovian matrix. 
  xml outputs can be activated to save the associated trees.
*/
class PhasedPMarkov : public PhasedMarkov
{
 protected : 
  //! Parcimonious Context Trees
  vector<pmm_forest*> _p_f;
  
 public:
  //! Constructor 1 from a SequenceSet
  /*!
    \param part associated partition
    \param  seqset a set of sequences for estimation
    \param phase phase 
    \param initial_phase phase of the first element of each sequence
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element, and for each phase (separated by a "#Phase i")
    \param motif_prior activates a weight-function on the priors, weight proportionnal on each motif in the tree
    \param penalty penalty on the leaves number, by default 0  
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  PhasedPMarkov( Partition& part,
		 const SequenceSet & seqset,
		 short phase, short initial_phase = 0,
		 const string& prior_alpha_file = string(),
		 bool motif_prior = true,
		 double penalty = 0.,
		 const string & xmlfile = string() )
    :  PhasedMarkov(  seqset.tell_alphabet_size(), seqset.tell_order(), phase )
  { 
    seqset.count_p_occurencies( phase, initial_phase );
    if (!this->load_prior_alpha( prior_alpha_file ))
      this->default_prior_alpha();
    for (int i=0; i<_phase; i++)
      _p_f.push_back( new pmm_forest( part, _size, _order, _prior_alpha[i], motif_prior, penalty ) );
    select( seqset.get_p_count(), true, seqset.get_translator(), xmlfile );
  }

  //! Constructor 2 from a Sequence
  /*!
    \param part associated partition       
    \param seq sequence for estimation
    \param phase phase 
    \param initial_phase phase of the first element of each sequence
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element, and for each phase (separated by a "#Phase i")
    \param motif_prior activates a weight-function on the priors, weight proportionnal on each motif in the tree
    \param penalty penalty on the leaves number, by default 0  
    \param trans a Translator is required only for the xml saving
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  PhasedPMarkov( Partition& part,
		 const Sequence & seq,
		 short phase, short initial_phase = 0,
		 const string& prior_alpha_file = string(),
		 bool motif_prior = true,
		 double penalty = 0., 
		 const Translator & trans = Translator(),
		 const string & xmlfile = string() )
    : PhasedMarkov(  seq.tell_alphabet_size(), seq.tell_order(), phase )
  {
    seq.count_p_occurencies( phase, initial_phase );
    if (!this->load_prior_alpha( prior_alpha_file ))
      this->default_prior_alpha();
    for (int i=0; i<_phase; i++)
      _p_f.push_back( new pmm_forest( part, _size, _order, _prior_alpha[i], motif_prior, penalty ) ); 
    select( seq.get_p_count(), true, trans, xmlfile );
  }
  
  //! Constructor 3 from a coded-word count
  /*!
    \param part associated partition       
    \param count count of all the coded word(base size) of size order+1 for each phase, for estimation
    \param size alphabet size
    \param order markovian order associated to the word count
    \param phase phase 
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element, and for each phase (separated by a "#Phase i")
    \param motif_prior activates a weight-function on the priors, weight proportionnal on each motif in the tree
    \param penalty penalty on the leaves number, by default 0  
    \param trans a Translator is required only for the xml saving
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  PhasedPMarkov( Partition& part,
		 unsigned long * * count,
		 short size, short order,
		 short phase, 
		 const string& prior_alpha_file = string(),
		 bool motif_prior = true,
		 double penalty = 0.,
		 const Translator & trans = Translator(),
		 const string & xmlfile = string() )
    :  PhasedMarkov( size, order, phase )
  { 
    if (!this->load_prior_alpha( prior_alpha_file ))
      this->default_prior_alpha();
    for (int i=0; i<_phase; i++)
      _p_f.push_back( new pmm_forest( part, _size, _order, _prior_alpha[i], motif_prior, penalty ) );  
    select( count, false, trans, xmlfile );
  }

  //! Constructor 4 from a file containing coded-word count
  /*!
    \param part associated partition
    \param count_file file with count
    \param size alphabet size
    \param order markovian order associated to the word count
    \param phase phase 
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element, and for each phase (separated by a "#Phase i")
    \param motif_prior activates a weight-function on the priors, weight proportionnal on each motif in the tree
    \param penalty penalty on the leaves number, by default 0  
    \param trans a Translator is required only for the xml saving
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  PhasedPMarkov( Partition& part,
		 const string & count_file,
		 short size, short order,
		 short phase, 
		 const string& prior_alpha_file = string(),
		 bool motif_prior = true,
		 double penalty = 0.,
		 const Translator & trans = Translator(),
		 const string & xmlfile = string() )
    :  PhasedMarkov( size, order, phase )
  { 
    unsigned long ** count = new unsigned long*[_phase];
    for (short p=0; p<_phase; p++ )
      count[p] = new unsigned long[_nPi];    
    if (! this->load_count( count_file, count )){
      cerr<<"Bad count file"<<endl;
      exit(1);
    }

    if (!this->load_prior_alpha( prior_alpha_file ))
      this->default_prior_alpha();
    for (int i=0; i<_phase; i++)
      _p_f.push_back( new pmm_forest( part, _size, _order, _prior_alpha[i], motif_prior, penalty ) );
    select( count, false, trans, xmlfile );

    for (short p=0; p<_phase; p++ )
      delete[] count[p];
    delete [] count;
  }
  
  //! Basic Constructor 5. No estimation.
  /*! 
    \param part associated partition          
    \param size alphabet size
    \param order markovian order associated to the word count
    \param phase phase 
    \param prior_alpha_file file containing the alpha for the a priori law, one value per alphabet element, and for each phase (separated by a "#Phase i")
    \param motif_prior activates a weight-function on the priors, weight proportionnal on each motif in the tree
    \param penalty penalty on the leaves number, by default 0 
    \param alloc true for matrices memory allocation
  */
  PhasedPMarkov( Partition& part,
		 short size,
		 short order,
		 short phase, 
		 const string& prior_alpha_file = string(),
		 bool motif_prior = true,
		 double penalty = 0.,
		 bool alloc = true )
    : PhasedMarkov( size, order, phase, alloc )
  {
    if (!this->load_prior_alpha( prior_alpha_file ))
      this->default_prior_alpha();
    for (int i=0; i<_phase; i++)
      _p_f.push_back( new pmm_forest( part, _size, _order, _prior_alpha[i], motif_prior, penalty ) );
  }

  //! performs the A Posteriori Maximisation
  /*! 
    \param count count of all the coded word(base size) of size order+1 for each phase, for estimation
    \param decal_required necessary when using a count of word from 1-word to (_max_depth+1)-word
    \param trans a Translator is required for xml tree saving (if xml2 activated)
    \param xmlfile xmlfile for tree saving (if xml2 activated)    
  */
  void select( unsigned long * * count, bool decal_required,
	       const Translator & trans = Translator(),
	       const string & xmlfile = string()  )
  {
   _nb_param = 0;
   long decal = 0;
   if (decal_required)
     decal = _jump;
   
#ifdef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     xmlDocPtr doc=xmlNewDoc(BAD_CAST "1.0");
     xmlNodePtr root_node=xmlNewNode(NULL,BAD_CAST "treeset");
     xmlDocSetRootElement(doc,root_node); 
     for (int i=0; i<_phase; i++){
	 pmm_tree p_t = _p_f[i]->select( count[i]+decal );
	 
       p_t.tree_to_matrix( _Pis[i] );
       _nb_param += p_t.nb_leaves() * (_size-1);

       std::stringstream treenamestream;
       treenamestream <<"phase "<<i;
       p_t.save( trans, root_node, treenamestream.str() );


     }
     xmlSaveFormatFileEnc(xmlfile.c_str(),doc,"ISO-8859-1",1);
     xmlFreeDoc(doc); 
   }
   else{
     for (int i=0; i<_phase; i++){

       pmm_tree p_t = _p_f[i]->select( count[i]+decal );
       p_t.tree_to_matrix( _Pis[i] );
       _nb_param += p_t.nb_leaves() * (_size-1);

     }
   }
#endif
#ifndef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     cerr<<"in PhasedPMarkov: no libxml2 detected to fill "<<xmlfile<<endl;
   }
   for (int i=0; i<_phase; i++){    

     pmm_tree p_t = _p_f[i]->select( count[i]+decal );
     p_t.tree_to_matrix( _Pis[i] );
     _nb_param += p_t.nb_leaves() * (_size-1);
       
     
   }
#endif
 }


  /*!
    \param tseq_train sequence(s) (set) for the training step
    \param initial_phase_train phase of the first element of each sequence
    \param tseq_eval sequence(s) (set) for the evaluation step
    \param initial_phase_eval phase of the first element of each sequence
  */
  template <class TSeq1, class TSeq2>
    double mean_post_log_likelihood( const TSeq1& tseq_train, const TSeq2& tseq_eval,
				     short initial_phase_train = 0, short initial_phase_eval = 0 )
  {
    tseq_train.count_p_occurencies(_phase, initial_phase_train);
    tseq_eval.count_p_occurencies(_phase, initial_phase_eval);
    return this->mean_post_log_likelihood( tseq_train.get_p_count(), true,
					   tseq_eval.get_p_count(), true ); 
  }

  //! return the mean post likelihood over the parameters and the trees
  /*! 
    \param count_train count of all the coded word(base size) of size order+1 for each phase, for training step
    \param decal_required_t necessary when using a count of word from 1-word to (_max_depth+1)-word
    \param count_eval count of all the coded word(base size) of size order+1 for each phase, for evaluation step
    \param decal_required_e necessary when using a count of word from 1-word to (_max_depth+1)-word
  */
  double mean_post_log_likelihood( unsigned long * * count_train, bool decal_required_t,
				   unsigned long * * count_eval, bool decal_required_e )
  {
    double mean_logv = 0;
    for (int i=0; i<_phase; i++){             
      if (decal_required_t)
	_p_f[i]->extended_tree( count_train[i]+_jump );
      else
	_p_f[i]->extended_tree( count_train[i] );
      if (decal_required_e)
	mean_logv += _p_f[i]->mean_post_log_likelihood( count_eval[i]+_jump );
      else
	mean_logv += _p_f[i]->mean_post_log_likelihood( count_eval[i] );
    }
    return mean_logv;
  }

  /*!
    \param tseq_eval sequence(s) (set) for the evaluation step
    \param initial_phase_eval phase of the first element of each sequence
  */
  template <class TSeq>
    double mean_post_log_likelihood( const TSeq& tseq_eval,short initial_phase_eval = 0 )
    {     
      tseq_eval.count_p_occurencies(_phase, initial_phase_eval);
      return this->mean_post_log_likelihood( tseq_eval.get_p_count(), true );
    }
  
  

  //! return the mean post likelihood over the parameters and the trees
  /*! 
    \param count_eval count of all the coded word(base size) of size order+1 for each phase, for evaluation step
    \param decal_required_e necessary when using a count of word from 1-word to (_max_depth+1)-word
  */
  double mean_post_log_likelihood( unsigned long * * count_eval, bool decal_required_e )
  {
    double mean_logv = 0;
    for (int i=0; i<_phase; i++){              
      if (decal_required_e)
	mean_logv += _p_f[i]->mean_post_log_likelihood( count_eval[i]+_jump );
      else
	mean_logv += _p_f[i]->mean_post_log_likelihood( count_eval[i]);
    }
    return mean_logv;
  }

  //! return the mean post likelihood over the parameters and the trees
  double mean_post_log_likelihood(){
    double sum = 0.;
    for (short i=0; i<_phase;i++){
      _p_f[i]->extended_tree();
      sum += _p_f[i]->mean_post_log_likelihood( );
    }
    return sum;
  }
  

  
 //---------------------------------------
  //! compute the mean posterior likelihood over the parameters
  /*!
    \param tseq_eval sequence(s) (set) for the evaluation step
    \param initial_phase_eval phase of the first element of each sequence
  */
  template <class TSeq>
    double post_log_likelihood( const TSeq& tseq_eval, 
				short initial_phase_eval = 0 )
  {    
    if ( tseq_eval.tell_order() != _order ){
      cerr << "Order incompatibility beetween  sequence and model " << endl;
      exit(-1);
    }
    // A FAIRE attention seq > order...
    
    tseq_eval.count_p_occurencies(_phase, initial_phase_eval);     
    
    return post_log_likelihood( tseq_eval.get_p_count(), true );
  }

  //! compute the mean posterior likelihood over the parameters
  /*! 
    \param count_eval count of all the coded word(base size) of size order+1 for each phase, for evaluation step
    \param decal_required_e necessary when using a count of word from 1-word to (_max_depth+1)-word
  */
  double post_log_likelihood( unsigned long * * count_eval, bool decal_required_e  )
  {
    double sum=0.;
  
    for (short p=0; p<_phase; p++){
      if (decal_required_e)
	sum += _p_f[p]->post_log_likelihood( count_eval[p]+_jump );
      else
	sum += _p_f[p]->post_log_likelihood( count_eval[p] );
    }
    return sum;
  }


  //---------------------------------------
  //! draws a model
  /*! 
    \param count count of all the coded word(base size) of size order+1 for each phase, for estimation
    \param decal_required necessary when using a count of word from 1-word to (_max_depth+1)-word
    \param r gsl random generator
    \param trans a Translator is required for xml tree saving (if xml2 activated)
    \param xmlfile xmlfile for tree saving (if xml2 activated)    
  */
  void draw( unsigned long * * count, bool decal_required,
	     gsl_rng * r,
	     const Translator & trans = Translator(),
	     const string & xmlfile = string() )
  {  
   _nb_param = 0;   
#ifdef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     xmlDocPtr doc=xmlNewDoc(BAD_CAST "1.0");
     xmlNodePtr root_node=xmlNewNode(NULL,BAD_CAST "treeset");
     xmlDocSetRootElement(doc,root_node); 
     for (int i=0; i<_phase; i++){
       if (decal_required)
	 _p_f[i]->extended_tree( count[i]+_jump );
       else 
	 _p_f[i]->extended_tree( count[i] );
	 
       pmm_tree p_t = _p_f[i]->draw(r);                
       p_t.tree_to_matrix( _Pis[i] );
       _nb_param += p_t.nb_leaves() * (_size-1);

	 std::stringstream treenamestream;
	 treenamestream <<"phase "<<i;
	 p_t.save( trans, root_node, treenamestream.str() );
     }
     xmlSaveFormatFileEnc(xmlfile.c_str(),doc,"ISO-8859-1",1);
     xmlFreeDoc(doc); 
   }
   else{
     for (int i=0; i<_phase; i++){

       if (decal_required)
	 _p_f[i]->extended_tree( count[i]+_jump );
       else
	  _p_f[i]->extended_tree( count[i] );
       pmm_tree p_t = _p_f[i]->draw(r);          
       p_t.tree_to_matrix( _Pis[i] );
       _nb_param += p_t.nb_leaves() * (_size-1);
    
     }
   }
#endif
#ifndef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     cerr<<"in PhasedPMarkov: no libxml2 detected to fill "<<xmlfile<<endl;
   }
   for (int i=0; i<_phase; i++){        
     if (decal_required)
       _p_f[i]->extended_tree( count[i]+_jump );
     else
       _p_f[i]->extended_tree( count[i] );
     pmm_tree p_t = _p_f[i]->draw(r);        
     p_t.tree_to_matrix( _Pis[i] );
     _nb_param += p_t.nb_leaves() * (_size-1);
     
   }
#endif

   if (_Mus != NULL)
     this->compute_stat_laws( true );
  }


  //! draws a model
  /*!   
    \param r random generator  
    \param trans a Translator is required for xml tree saving (if xml2 activated)
    \param xmlfile xmlfile for tree saving (if xml2 activated)    
  */
  void draw( gsl_rng * r,
	     const Translator & trans = Translator(),
	     const string & xmlfile = string()  )
  {
    if (!_p_f[0]->extended_ok()){
      cerr<<"PhasedPMarkov::draw = count required for a first draw"<<endl;
      exit(1);
    }

   _nb_param = 0;   
#ifdef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     xmlDocPtr doc=xmlNewDoc(BAD_CAST "1.0");
     xmlNodePtr root_node=xmlNewNode(NULL,BAD_CAST "treeset");
     xmlDocSetRootElement(doc,root_node); 
     for (int i=0; i<_phase; i++){
       
       pmm_tree p_t = _p_f[i]->draw(r);                
       p_t.tree_to_matrix( _Pis[i] );
       _nb_param += p_t.nb_leaves() * (_size-1);

	 std::stringstream treenamestream;
	 treenamestream <<"phase "<<i;
	 p_t.save( trans, root_node, treenamestream.str() );
     }
     xmlSaveFormatFileEnc(xmlfile.c_str(),doc,"ISO-8859-1",1);
     xmlFreeDoc(doc); 
   }
   else{
     for (int i=0; i<_phase; i++){
       
       pmm_tree p_t = _p_f[i]->draw(r);     
       p_t.tree_to_matrix( _Pis[i] );   
       _nb_param += p_t.nb_leaves() * (_size-1);

     }
   }
#endif
#ifndef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     cerr<<"in PhasedPMarkov: no libxml2 detected to fill "<<xmlfile<<endl;
   }
   for (int i=0; i<_phase; i++){        
     
     pmm_tree p_t = _p_f[i]->draw(r);        
     p_t.tree_to_matrix( _Pis[i] );
     _nb_param += p_t.nb_leaves() * (_size-1); 
    
   }
#endif   
   
   if (_Mus != NULL)
     this->compute_stat_laws( true );
  }


  //! returns info on the number of leaves for each phase
  void info_nb_leaves() const{
    for (short p=0; p<_phase; p++){
      cout<<"Phase "<<p<<": "
	  <<_p_f[p]->current_pmm_tree().nb_leaves()<<" leaves"
	  <<endl;
    }
  }

  void set_penalty( double penalty ){
    for (int i=0; i<_phase; i++) 
      _p_f[i]->set_penalty( penalty);
  }

  void default_prior_alpha(){
    vector<double> tmp;
    tmp.assign(_size, 1.);
    _prior_alpha.assign(_phase, tmp);
  }


  //! Destructor
  ~PhasedPMarkov(){
    for (int i=0; i<_phase; i++) 
      if (_p_f[i])
	delete _p_f[i];
  } 
};
#endif
