/* seqpp/PhasedVLMarkov.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
 \file
  \author Vincent Miele
  \brief  Phased Variable Length Markov model

   01/2004
*/

#ifndef SEQPP_PHASEDVLMARKOV_H
#define SEQPP_PHASEDVLMARKOV_H

#include <seqpp/PhasedMarkov.h>
#include <seqpp/vlm_tree.h>


/**
 * \ingroup General_Use_Markov
 */

/*!  \class PhasedVLMarkov seqpp/PhasedVLMarkov.h
\brief  Phased Variable Length Markov modelling, estimation and simulation.

  PhasedVLMarkov is a PhasedMarkov object with a different estimation step.
  This object performs the estimation with the Variable Length Markov algorithm 
  and then transfoms, once per phase, the variable length context tree in a markovian matrix.
  One difference is that a cutoff is required for the selection of the tree.
*/

class PhasedVLMarkov : public PhasedMarkov
{
 protected :
  //! Variable Length Context Trees
  vlm_tree * _vlmtree;

 public: 
  //! Constructor 1 from a SequenceSet or a Sequence
  /*!
    \param  tseq  a (set of) sequence(s) for estimation
    \param init initial law in the leaves
    \param cutoff tree selection cutoff
    \param phase phase 
    \param initial_phase phase of the first element of each sequence
  */
  template <class TSeq>
    PhasedVLMarkov( const TSeq & tseq,
		    vector<double> & init, double cutoff,
		    short phase, short initial_phase = 0 )
    :  PhasedMarkov(  tseq.tell_alphabet_size(), tseq.tell_order(), phase )
  {
    _vlmtree = new vlm_tree( tseq.get_coder(),
			    _size,			    
			    _order,
			    init );
    tseq.count_p_occurencies( phase, initial_phase );
    _nb_param = 0;
    for (int i=0; i<phase; i++){
      _vlmtree->estimate_context( tseq.get_count(i),
				  cutoff );
      _vlmtree->tree_to_matrix( _Pis[i] );
      _nb_param += _vlmtree->nb_leaves() * (_size-1); 
    }
    delete _vlmtree;
  }
  
  //! Constructor 2 from a SequenceSet or a Sequence with random init 
  /*!
    \param  tseq  a (set of) sequence(s) for estimation    
    \param cutoff tree selection cutoff
    \param phase phase 
    \param initial_phase phase of the first element of each sequence
  */
  template <class TSeq>
    PhasedVLMarkov( const TSeq & tseq,
		    double cutoff,
		    short phase, short initial_phase = 0 )
    :  PhasedMarkov(  tseq.tell_alphabet_size(), tseq.tell_order(), phase )
  {
    vector<double> init;
    for (short i = 0; i<_size; i++)
      init.push_back( 1./_size );
    
    _vlmtree = new vlm_tree( tseq.get_coder(),
			     _size,			    
			    _order,
			    init );
    tseq.count_p_occurencies( phase, initial_phase );
    _nb_param = 0;
    for (int i=0; i<phase; i++){
      _vlmtree->estimate_context( tseq.get_count(i),
				  cutoff );
      _vlmtree->tree_to_matrix( _Pis[i] );
      _nb_param += _vlmtree->nb_leaves() * (_size-1);  
    }
    
    delete _vlmtree;
  }

  //! destructor
  ~PhasedVLMarkov(){};
};
#endif
