/* seqpp/PrimaryCount.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
  \file
  \author Vincent Miele
  \brief Abstract class designed for the (coded)word count 

   23/04/2003
*/


#ifndef SEQPP_PRIMARYCOUNT_H
#define SEQPP_PRIMARYCOUNT_H

#include <seqpp/Coder.h>

/**
 * \ingroup General_Use_Sequence
 */

/*!  \class PrimaryCount seqpp/PrimaryCount.h 
  \brief Abstract class designed for the (coded)word count 

  This object is based on integer codes to store words as required for a markov modelling (see Sequence).
  The words are coded in alphabet_size base, coding the 1-token words, then the 2-tokens words, the 3-tokens words...
  Let call jump(i) the first code of a i-tokens words, a PrimaryCount object is designed to count all words:
  \code
  (example with alphabet AGCT and order=2)
  0              first word ( 0 -> A)
  ..
  ..             1-token words (from A to T)
  ..
  jump(1)        (4 -> AA)
  ..
  ..             2-tokens words (from AA to TT)
  ..
  jump(2)        (20 -> AAA)
  ..
  ..             3-tokens words (from AAA to TTT)
  ..
  ..
  ..             last word (TTT)
  jump(order+1)  (64 -> /end)
  \endcode
  The count can also be phased.
*/
class PrimaryCount
{
 protected:
   //-----------------------
   //! Order of the model
   short _order;
   //! nber of codes/words taking into interest
   long _nbvalue;
   //! number of invalid negative code
   short _nbinv;   

  //! markovian coder associated to the order of the sequence
   Coder * _coder;
   //! true if _coder is build in the class
   bool _coder_in;
  
   //-----------------------
   //! count of each code/word, even invalid (-1)
   unsigned long * * _count;
   //! if the count is available
   bool _count_ok; 
   //! phase for the count
   short _phase;
    
   //! initialize the count
   void init_count() const;

   //----------------------- 
   //! return the max length for the count 
   virtual unsigned long tell_max_count() const = 0;  
   //! compute the count
   virtual void compute_count( unsigned long * * extern_count, 
			       short phase,
			       short initial_phase,
			       unsigned long beg,
			       unsigned long end ) const = 0;
   //----------------------- 
			  
 public:

  PrimaryCount(){ 
    //cout<<"default constr pr.count"<<endl;
    _order = -1;_nbvalue=0;_nbinv=0;_coder=NULL;_coder_in=false;
    _count=NULL;_count_ok=false;_phase=0;
  };


   //! destructor
   virtual ~PrimaryCount(){
     //cout<<"debut destruct. pr.count"<<endl;
     if (_coder_in)
       delete _coder;
     //cout<<"c ici"<<endl;
     this->clear_count();
     //cout<<"fin destruct. pr.count"<<endl;
   }

   //-----------------------  
   //! clear the count
   void clear_count() const;

  //! put all counts down to 0
  void null_count() const;

   //! returns the markovian order
   short tell_order() const{
	   return _order;
   }
   //! returns the nber of codes/words taking into interest
   int tell_nb_value() const{
     return _nbvalue;
   }    
   //! returns the jump to the codes of _order+1 letters
   long tell_jump() const{
     return _coder->tell_jump();
   }

  //! returns the vector of the jumps to the codes of i letters, 0<i<=order+2
  long * get_jump() const{
     return _coder->get_jump();
   }
   
   //! allows a const access to the coder
   Coder &  get_coder() const{
     return *_coder;
   }
  
  //-----------------------
  //! checks if the count is available
  bool is_count_ok() const{
    return _count_ok;
  }
  
  //! get the total count, in the phased case
  unsigned long * * get_p_count() const{
    return _count;
  } 
  
  //! get the total count in a phase p, or in the unphased case
  /*! 
    \param p selected phase, default 0 if unphased case.   
  */
  unsigned long * get_count( short p = 0 ) const{
     return _count[p];
   }

  //! returns nb of phase
  short tell_phase() const{
    return _phase;
  }
  
  //--------------------------------------------- 
  //! \brief  Phased count the number of occurencies of all the nbvalue codes (in the sub-sequence [beg...end])... beware: beg>=0 but not >0 
  /*!
    \param phase phase
    \param initial_phase phase of the first element of associated sequence
    \param beg the sub-sequence [beg...
    \param end the sub-sequence ...end]
  */
   void count_p_occurencies( short phase,
			     short initial_phase,
			     unsigned long beg,
			     unsigned long end ) const{
     compute_count( const_cast< PrimaryCount* >( this )->_count,
		    phase, initial_phase, beg, end );       
   }
  
  //! \brief  Phased extern count the number of occurencies of all the nbvalue codes (in the sub-sequence [beg...end])... beware: beg>=0 but not >0 
  /*!
    \param extern_count extern count storage
    \param phase phase
    \param initial_phase phase of the first element of associated sequence
    \param beg the sub-sequence [beg...
    \param end the sub-sequence ...end]
  */
  void count_p_occurencies( unsigned long * * extern_count, 
			    short phase,
			    short initial_phase,
			    unsigned long beg,
			    unsigned long end ) const{
    compute_count( extern_count, phase, initial_phase, beg, end );
   }
    
  //! \brief  Phased extern count the number of occurencies of all the nbvalue codes in the whole sequence  
  /*!
    \param extern_count extern count storage
    \param phase phase
    \param initial_phase phase of the first element of associated sequence
  */
  void count_p_occurencies( unsigned long * * extern_count, 
			    short phase,
			    short initial_phase = 0 ) const{
    count_p_occurencies(  extern_count, phase, initial_phase, 0, 0 );
  }
  
  //! \brief  Phased count the number of occurencies of all the nbvalue codes in the whole sequence  
  /*! 
    \param phase phase
    \param initial_phase phase of the first element of associated sequence
  */
  void count_p_occurencies( short phase,
			    short initial_phase = 0 ) const{
    count_p_occurencies( phase, initial_phase, 0, 0 );
  } 
  
  //---------------------------------------------
   //! \brief Extern count the number of occurencies of all the nbvalue codes (in the sub-sequence [beg...end])... beware: beg>=0 but not >0 
  /*!
    \param beg the sub-sequence [beg...
    \param end the sub-sequence ...end]
  */
  void count_occurencies( unsigned long beg,
 			  unsigned long end ) const {
    count_p_occurencies(1,0,beg,end);
  }

  //! \brief Count the number of occurencies of all the nbvalue codes in the whole sequence  
  void count_occurencies( ) const{
    count_p_occurencies( 1, 0, 0, 0 );
  } 
 
  //---------------------------------------------
  //! extract the number of occurencies of code in phase
  unsigned long tell_p_occurencies( long code, short numphase ) const;

  //! extract the number of occurencies of obs
  unsigned long tell_occurencies( long code ) const {
    return tell_p_occurencies( code, 0 ) ;
  }
};
#endif
