/* seqpp/PrimarySequenceSet.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*!
  \file
  \author Vincent Miele
  \brief Generic set of sequence objects

   23/04/2003
*/

#ifndef SEQPP_PRIMARYSEQUENCESET_H
#define SEQPP_PRIMARYSEQUENCESET_H

#include <seqpp/Sequence.h>
#include <seqpp/Translator.h>
#include <iomanip>

/**
 * \ingroup General_Use_Sequence
 */

/*! \class PrimarySequenceSet seqpp/PrimarySequenceSet.h
  \brief Class corresponding to a generic sequence set.
*/
template <class TSequence> class PrimarySequenceSet
{
  protected :
    
    //! Associated alphabet
    const Translator *_transl; 
    
  //! number of possible observable value
  short _nmodal;

  //! Number of sequences in the array
  int _nb_seq;
  
  //! Total length
  unsigned long _total_length;
    
  //! Maximum length of a sequence
  unsigned long _length_max;
    
  //! Array of sequences 
  TSequence * * _array_seq;
  
  public : 
  //! default constructor
  PrimarySequenceSet(){
    _transl=NULL;_nmodal=0;_nb_seq=0;_total_length=0;_length_max=0;_array_seq=NULL;
  }

  //! destructor
  virtual ~PrimarySequenceSet(){  
    //if (DEBUG>0)
    // cout<<">>>>>>>>>> Delete PrimarySequenceSet ----------"<<endl;      
    for ( int i=0 ; i<_nb_seq ; i++ )
      delete _array_seq[i];
    delete [] _array_seq;
    delete _transl;      
    //if (DEBUG>0)
    //cout<<">>>>>>>>>> END Delete PrimarySequenceSet ----------"<<endl;
  }

  //Methods --------------------------------
  
  //! allows a const access to the alphabet via a reference
  const Translator &  get_translator() const{
    return *_transl;
  }
  
  //! returns the alphabet size
  short tell_alphabet_size () const{
    return _transl->tell_alphabet_size() ;
  } 

  //! returns the nber of invalid codes
  short tell_nb_inv () const{
    return (*_transl).tell_nb_inv();
  }

  //----------------------------------------
  
  //! allows a const access to ith sequence via a reference
  TSequence &  get_sequence ( int i ) const{
    return *_array_seq[i];
  }
  //! allows a const access to ith sequence via a reference
  TSequence &  operator() ( int i ) const{
    return *_array_seq[i];
  }

  //! returns the number of sequences
  int tell_nb_sequence () const{
    return _nb_seq;
  }
  
  //! returns the total length
  virtual unsigned long tell_length ()const{
    return _total_length;
  }
   
  //! returns the length max
  unsigned long tell_length_max () const{
    return _length_max;
  }

  //! returns the length of sequence i
  unsigned long tell_length_seq( int i ) const{
    if ( (i>=_nb_seq) || (i<0) ){  // avant: (i>0)
      cout<<"PrimarySequenceSet::tell_length_seq: Out Of Sequences Index"
	  << "  i = " << i
	  <<endl; 
      return 0;
    }
    else 
      return _array_seq[i]->tell_length();
  }
   
  //! gives the name oh the ith file
  string tell_seq_name(int i) const{
    if (i>=_nb_seq){
      cout<<"PrimarySequenceSet::tell_seq_name: Out Of Sequences Index"<<endl; 
      return NULL;
    }
    else 
      return _array_seq[i]->tell_seq_name();
  }

  //! gives the name oh the ith file
  string tell_file_name( int i ) const{
    if (i>=_nb_seq){
      cout<<"PrimarySequenceSet::tell_file_name: Out Of Sequences Index"<<endl; 
      return NULL;
    }
    else 
      return _array_seq[i]->tell_file_name();
  } 

  //! compute the weight matrix associated to the sequences  wmat[token][position in the sequence]
  void weight_matrix( double ** wmat ) const{
  unsigned long ul, l = _array_seq[0]->tell_length();
  bool stop = false;
  int i=0, nbok, x;
  short size = _array_seq[0]->tell_alphabet_size();
  while( (!stop)&&(i<_nb_seq) ){
    if (_array_seq[i++]->tell_length() != l)
      stop = false;    
  }
  if (stop)
    cerr<<"for weigth matrix : sequence with different lengths!";
  else
    for (ul = 0; ul<l; ul++){
      nbok = _nb_seq;
      for (i=0; i<_nb_seq; i++){
	x = _array_seq[i]->tell_int(ul);
	if ( x<0 )
	  nbok--;
	else
	  wmat[x][ul]++;
      }
      for (i=0; i<size; i++)
	if (nbok>0)
	  wmat[i][ul] /= nbok;
    }
}  
}; 
#endif /* SEQPP_PRIMARYSEQUENCESET_H */
