/* seqpp/Sequence.h
 * 
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
  \file
  \author Vincent Miele - Florence Muri - Pierre Nicolas
  \brief Sequence class has been designed for Markovian analysis.

   23/04/2003
*/


#ifndef SEQPP_SEQUENCE_H
#define SEQPP_SEQUENCE_H

#include <seqpp/PrimarySequence.h>
#include <seqpp/PrimaryCount.h>
#include <seqpp/Coder.h>
#include <set>

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
/**
 * \ingroup General_Use_Sequence
 */
/*!  \class Sequence seqpp/Sequence.h 
  \brief The Sequence class has been designed for Markovian analysis of a single sequence.
 
  On each position, the markovian predictor is coded and store (see PrimaryCount):
\code
  Exple with the alphabet AGCT: 
  sequence position  .... 7  8  9 ...
  token              .... A  G  A ...
  code stored        ..........24
  \endcode

  Sequence object of integer values corresponding
  to the code of the word considered on a position t   
  for a Markov model of order _order... this code corresponds to
  the number of smaller words and the calculus in base |A| of this word, with A the alphabet.<BR>           
  ex : from A to GGG for order 2 with dna alphabet 
*/

class Sequence : public PrimarySequence, public PrimaryCount
{
 protected :
 
  short tell_int_speed( unsigned long pos ) const{
    return _obs[pos]%_nmodal;
  }
  //! return the max length for the count 
  virtual unsigned long tell_max_count() const{
    return _length;
  }
  virtual void compute_count( unsigned long * * extern_count, 
			      short phase,
			      short initial_phase,
			      unsigned long beg,
			      unsigned long end ) const;
 public :
   /*! \brief Constructor from a flow ifile from a fasta file
    
  on the position of the char '>', create an object until
  the next '>' generating his name by the concatenation
  of the sentence after the char '>'.
  The flow is or eof or on a '>' at the end.
  \param ifile iftream flow
  \param order markovian order
  \param transl reference to a Translator
  \param coder reference to a Coder
  \param name_file file name corresponding to ifile
  */
  Sequence( ifstream & ifile,
	    short order,
	    const Translator & transl,
	    Coder & coder,
	    const string & name_file = "noname" );

  // Constructors-destructor -------------
  //! minimal constructor
  /*!
    \param order markovian order
    \param length default length
    \param transl reference to a Translator
  */
  Sequence( short order, unsigned long length,
	    const Translator & transl );
  
  //! Constructor from a Fasta or Genbank file containing a single sequence
  /*!
    \param  name_file file name to open
    \param order markovian order    
    \param transl reference to a Translator
  */
  Sequence(  const  string &  name_file, short order,
	     const Translator & transl );
     
  //! Copy constructor
  Sequence( const Sequence & seq );
  
  //! Deduction constructor from an order 0-sequence
  /*
    \param seq order 0-sequence
    \param order new markovian order
  */
  Sequence( const Sequence & seq, short order );
  
  //! destructor
  virtual ~Sequence();

  //Methods --------------------------------
  //! Returns the int value of the token on position pos
  short tell_int(unsigned long pos) const;

  //---------------------------------------------
  //! Returns the code on position pos 
  long tell_code( unsigned long pos ) const{  
    return(_obs[pos]);
  }

  //! idem "tell_code" Returns the code on position pos 
  long & operator() ( unsigned long pos ) const{  
    return(_obs[pos]);
  }

  //! Returns the code on current position 
  long tell_code() const{
    return(_obs[_ipos]);
  }

  
  // Return the complementary code on position t    
  //Beware the complexity of this calculus !*/
  //    int tell_code_compl( long pos ) const{
  // return this->code_compl( _obs[pos] );
  //  }    
  //Return the complementary code on position t    
  //Beware the complexity of this calculus !*/
  //int tell_code_compl( )const{
  //  return this->code_compl( _obs[_ipos] );
  //}
  
  
  //! Returns the coded sequence as a vector
  const long * seq( ) const{
    return _obs;
  }
  //! Returns the coded sequence as a vector
  const long * get_code( ) const{
    return _obs;
  }  
  
 
  //--------------------------------------------
  //! simulate the observations from a transition matrix and a vector for initial _order-word proba
  /*
    \param m_trans markovian transition matrix
    \param v_init initial law
    \param r gsl random generator
    <br>GSL use exple:<br>
    \code
    const gsl_rng_type * T;
    // Choice a default generator and seed
    // from environment variables
    gsl_rng_env_setup();
    // New created instance of the generator
    T = gsl_rng_default;
    gsl_rng * r = gsl_rng_alloc (T);
    // Initialize/Seeds the random number generator
    gsl_rng_set( r, (long)getpid() );
    \endcode
  */
  void simule( const double * m_trans,
	       const double * v_init,
	       const gsl_rng * r );

  //! simulate the observations from a transition matrix, random initial law 
  /*
    \param m_trans markovian transition matrix    
    \param r gsl random generator 
  */
  void simule( const double * m_trans,
	       const gsl_rng * r ){
    simule( m_trans, NULL, r );
  }
  //! simulate the observations using the stat law the tokens composition
  /*
    \param v_statlaw stat_law law
    \param s_statlaw stat_law size
    \param r gsl random generator 
  */
  void simule_under_composition( const double * v_statlaw, long s_statlaw,
				 const gsl_rng * r );
  /*
    void simule( const Markov& m,
    const gsl_rng * r ){
    simule( m->getPi(), m->getMu(), r );
    }
  */

  //! simulate the observations from a vector of transition matrices and a vector for initial _order word proba
  void simule_phased(  short phase, 
		       const double ** m_trans,
		       const double * v_init,		      
		       const gsl_rng * r );


  //! simulate the observation on position from a alphabet size vector
  void simule( unsigned long position, 
	       double * proba, 
	       const gsl_rng * r );


  //---------------------------------------------
 
};

int Multinomial_1_Draw ( int dim, 
			 const double * param, const gsl_rng * r );
int Multinomial_1_Draw ( int dim, const gsl_rng * r );

#endif /* SEQPP_CSEQUENCE_H */
