/* seqpp/Translator.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
 *   \file
 *     \author Vincent Miele - Pierre Yves Bourguignon
 *       \brief General Alphabet class
 *
 *          23/04/2003
 *          
*/

#ifndef SEQPP_TRANSLATOR_H
#define SEQPP_TRANSLATOR_H

#include <seqpp/const.h>
using namespace s_s;
#include <iostream>
#include <fstream>
#include <string>
#include <map>
#include <vector>

using namespace std;


/**
 * \internal
 * \defgroup General_Use_Sequence Library Design: Classes dedicated to sequences/set of sequences/alphabet
 */


/**
 * \ingroup General_Use_Sequence
 */


/*! \class Translator seqpp/Translator.h
  \brief Alphabet designer tool.

  The key of the seq++ flexibility is the translator object. It defines a mapping between the alphabet and integer values. 
  In seq++, a letter of an alphabet, called a  token, can be made of one or more characters, for instance to manage n-uplets like codons. 
  It is associated to its rank integer position in the alphabet.<BR>
  A simple configuration file is required, containing
  synonymous tokens, complementary tokens (if exist) and labels. 
  A label aims to be the explicit name of a token. Moreover, invalid tokens can be declared and associated to negative values.<BR>
  
  Use a initial file containing same size(alphabet_mode characters) tokens (the letters of the alphabet)<BR>
  - on a same line, all the synonymous tokens (beware lower and upper case)<br>
  - (if necessary) on a line beginning with a "?", single characters or tokens (same size than others) to declare a class of invalid tokens. NB: a default class contains all the tokens not declared (neither belonging to alphabet nor to known invalids)<br>
  - (if necessary) their associated label (formally the common name) after a "->"
  - (if necessary) their complementary after a ":" <br>
  Beware the spaces between each tokens !<br>

  exple :<br>
  \code
  #codons
  ttt TTT -> F1
  ttc TTC -> F2
  tta TTA -> L1
  ...
  ggg GGG -> G4
  ? taa TAA tag TAG tga TGA  
  \endcode
  \code
  #dna
  a A : t T
  g G : c C
  c C : g G
  t T : a A
  ? n N
  \endcode 
  \code#secondary structures
  H -> HELIX
  B -> BETA
  C -> COIL
  \endcode 
  OR <br>
  Use an expression to parse ("line2parse") where all the same size ("mode") tokens
  are concatened, with a ":" to define the inverse complementary
  with same size tokens. No synonyms supported.
  exple : 
  \code
  acgt:tgca         # dna alphabet, mode = 1
  R1R2T1T2T3X1Y1Y2  # other alphabet, with mode = 2 (R1 is the first token...)
  \endcode
*/

class Translator
{
 public:

  //! Constructor from a configuration file
  /*!
    \param alpha_file configuration file
  */
  Translator ( const string & alpha_file );
  
  //! Constructor from an expression
  /*! 
    \param line2parse alphabet expression
    \param mode number of characters of a token
    \param case_sensivity  false if no case sensitivity required
  */
  Translator ( const string & line2parse, short mode,
	       bool case_sensivity = true );

  //! Copy constructor
  Translator( const Translator & t );
 
  //! Default constructor
  Translator() {
    _i2s=NULL;_i2lab=NULL;_Ci2i=NULL;
    _alphabet_size = 0;_alphabet_mode = 0;_nb_inv = 0;
   _complementary_ok = false;_label_ok = false;_tmp = NULL;
  }

  //! Destructor
  ~Translator ();

  //! Returns alphabet size
  short tell_alphabet_size () const{
    return _alphabet_size;
  }

  //! Return the number of invalid code
  short tell_nb_inv () const{
    return _nb_inv;
  }

  //! Return alphabet mode, the number of characters of a token
  short tell_alphabet_mode () const{
    return _alphabet_mode;
  }

  //! Checks if complementary codes are defined
  bool is_complementary_ok() const{
    return _complementary_ok;
  }

  //! Checks if labels are defined
  bool is_label_ok() const{
    return _label_ok;
  }
 
  //--------------------------------------------- 
  //--------------------------------------------- 
  //---------------------------------------------
  //---------------------------------------------
  //----------------------------------------------
  //!Returns the int code for a given char *
  /*!
    \param source pointer to the first character of a token to translate in int
  Beware that source must be the beginning of a word  of size _alphabet_mode, or -1 is returned 
  */
  short int char_to_int( const char * source ) const; 
  //!Returns the int code for a given string
  /*!
    \param source string-token to translate in int
  */
  short int str_to_int( const string& source ) const;

  /*! \brief Returns the complementary int code for a given char *

    \param source pointer to the first character of a token to complementary translate in int
    Beware that source must be the beginning of a word  of size _alphabet_mode, or -1 is returned 
  */
  short complementary_char_to_int(const char * source ) const;
  /*! \brief Returns the complementary int code for a given string

    \param  source string-token to complementary translate in int
  */
  short int complementary_str_to_int( const string& source ) const;


  //! Returns the complementary int code for a given code
  /*!
    \param  source integer code
  */
  short complementary_int(short source) const{
    return _Ci2i[source];
  }

  //---------------------------------------------
  /*! \brief Initialize dest to the word  corresponding to the int source
  
    \param dest must be allocated with _alphabet_mode+1 space, empty word if bad query.
    \param source integer to translate
  */
  void int_to_char(char * dest,
		   short source) const;
  //! returns the word corresponding to the int source
  /*
    \param source integer to translate
  */
  string int_to_str( short source ) const;


  //---------------------------------------------
  /*! \brief Initialize dest to the word corresponding to the complementary of word source
    
    \param dest must be allocated with _alphabet_mode+1 space, empty word if bad query.
    \param source pointer to the first position of the source token 
  */
  void complementary_char(char * dest,
			  const char * source) const{
    int_to_char( dest, complementary_char_to_int(source) );
  }
  //! returns the token corresponding to the inverse complementary of a token source
  /*
    \param source source token
  */
  string complementary_str( const string& source) const{
    return int_to_str( complementary_str_to_int( source ) );
  }

  //---------------------------------------------  
  //! returns the string corresponding to the complementary of int source
  /*
    \param source integer to complementary translate
  */
  string complementary_int_to_str( short source ) const{ 
    return int_to_str( _Ci2i[source] );  				   
  }
    
  //--------------------------------------------- 
  //--------------------------------------------- 
  //---------------------------------------------
  //---------------------------------------------
  //! returns the inverse complementary string-word of source    
  string complementary_strw( const string& wordsource) const;
  
  //---------------------------------------------
  /*! \brief a char-word <- an int-vect

  \param worddest word (must be allocated sizeword*_alphabet_mode + 1 space) corresponding to the sequential translation of source
  \param vectsource vector of integer to translate
  \param sizeword size of the vectsource to translate
  */
  void vect_to_word( char * worddest,
		     const short * vectsource,
		     short sizeword ) const;
  //! a char-word <- an int-vect
  /*
    \param worddest word (must be allocated sizeword*_alphabet_mode + 1 space) corresponding to the sequential translation of source
    \param vectsource vector of integer to translate
  */
  void vect_to_word( char * worddest,
		       const vector<short>& vectsource ) const;
  //! \brief a string-word <- an int-vect
  /*
    \param vectsource vector of integer to translate
    \param sizeword size of the vectsource to translate
  */
  string vect_to_strw(  const short * vectsource,
			short sizeword ) const;
  //! \brief a string-word <- an int-vect
  /*
    \param vectsource vector of integer to translate   
  */
  string vect_to_strw(  const vector<short>& vectsource ) const;


  //---------------------------------------------
  /*! \brief a char-word -> an int-vect
      
  \param vectdest integer vector (must be sizeword-allocated)
  \param wordsource character word to sequentially translate
  \param sizeword size of the word 
  */
  void word_to_vect( short * vectdest,
		     const char * wordsource,
		     short sizeword ) const;
  //! a char-word -> an int-vect
  /*
    \param vectdest integer vector 
    \param wordsource character word to sequentially translate 
  */
  void word_to_vect( vector<short>& vectdest,
		     const char * wordsource ) const;
  
  //! a string-word -> an int-vect  
   //! a char-word -> an int-vect
  /*
    \param vectdest integer vector 
    \param wordsource string word to sequentially translate 
  */
  void strw_to_vect( vector<short>& vectdest,
		      const string& wordsource ) const;

  //---------------------------------------------
  /*! \brief  a char-word -> an int code (base alphabet_size)
    
  \param wordsource pointer to the character word to code
  \param l its length
  */
  long word_to_coded_int( const char * wordsource, int & l ) const;

  /*! \brief  a string-word -> an int code (base alphabet_size)
    
  \param wordsource word to code
  */
  long strw_to_coded_int( const string& wordsource ) const;

  //worddest is a char-word associated to a code 
  //int size_word_for_coded_int( long codesource ) const; impossible  
  //"worddest" must be allocated in
  // (size_word_for_coded_int(codesource)*_alphabet_mode + 1) space
  //void coded_int_to_word( char * worddest, long codesource ) const;

  //! Returns the complementary int code for a given coded int
  /*
    \param codesource code to complementary code (in base alphabet_size) 
  */
  long complementary_coded_int( long codesource , short sizeword ) const;

  //---------------------------------------------
  //! checks if source is invalid
  short isInvalid( const char * source ) const;
  //! checks if source is invalid
  short isInvalid( const string& source ) const;

  //---------------------------------------------
  //! returns the label corresponding to the source
  /*
    \param source corresponding integer value 
  */
  string int_to_label( short source ) const{
    if ( (source<_alphabet_size) && (source>=-_nb_inv) )
      return _i2lab[source];
    else return "";
  }
  
  //! a string-word of label <- an int-vect
  /*
    \param vectsource vector of integer to ewtract the labels 
  */
  string vect_to_labelstrw( const vector<short>& vectsource ) const;

 protected:
 
   map<string, short> _s2i;
   string * _i2s;
   string * _i2lab;
   short * _Ci2i;

   map<string, short> _INVs2i;
   map<char, short> _INVc2i;
   
   //! number of tokens describing the alphabet
   short _alphabet_size;
   //! number of characters of each token
   short _alphabet_mode;
   //! number of invalid tokens
   short _nb_inv;

   //! checks if complementary is defined
   bool _complementary_ok;
   //! checks if labels are defined
   bool _label_ok;
 
  // tools
  char * _tmp;
  map<string, short>::const_iterator  _iter_si;
  map<char, short>::const_iterator _iter_ci;

  short file2nbinv( ifstream & f, short & nbtot );
  short line2nbtoken( const string & line2parse );
};


#endif //SEQPP_TRANSLATOR_H
