/* seqpp/VLMarkov.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
 \file
  \author Vincent Miele
  \brief  Variable Length Markov model
   01/2004
*/

#ifndef SEQPP_VLMARKOV_H
#define SEQPP_VLMARKOV_H

#include <seqpp/Markov.h>
#include <seqpp/PhasedVLMarkov.h>
#include <seqpp/vlm_tree.h>

/**
 * \ingroup General_Use_Markov
 */

/*!  \class VLMarkov seqpp/VLMarkov.h
\brief   Variable Markov modelling, estimation and simulation.

This is a special case of a phased Variable LengthMarkov [PhasedVLMarkov] model when only one phase is considered, and also a Markov object.
*/

class VLMarkov : public Markov
{
 protected :
  //! Variable Length Context Tree
  vlm_tree * _vlmtree;

 public: 
  //! Constructor 1 from a SequenceSet or a Sequence
  /*!
    \param tseq  a (set of) sequence(s) for estimation
    \param init initial law in the leaves
    \param cutoff tree selection cutoff
  */
  template <class TSeq>
    VLMarkov( const TSeq & tseq,
	      vector<double> & init, double cutoff )
    : Markov( tseq.tell_alphabet_size(), tseq.tell_order() )
  {   
    _vlmtree = new vlm_tree( tseq.get_coder(),
			     _size,			    
			     _order,
			     init );
    tseq.count_occurencies( );
    _vlmtree->estimate_context( tseq.get_count(),
				cutoff );
    _vlmtree->tree_to_matrix( _Pi );
    _nb_param = _vlmtree->nb_leaves() * (_size-1); 

    delete _vlmtree; 
  }
  
  //! Constructor 2 from a SequenceSet or a Sequence with random init 
  /*!
    \param  tseq  a (set of) sequence(s) for estimation    
    \param cutoff tree selection cutoff
  */
  template <class TSeq>
    VLMarkov( const TSeq & tseq,
	      double cutoff )
    : Markov( tseq.tell_alphabet_size(), tseq.tell_order() )
  {
    vector<double> init;
    for (short i = 0; i<_size; i++)
      init.push_back( 1./_size );
    
    _vlmtree = new vlm_tree( tseq.get_coder(),
			     _size,			    
			     _order,
			     init );
    tseq.count_occurencies( );
    _vlmtree->estimate_context( tseq.get_count(),
				cutoff );
    _vlmtree->tree_to_matrix( _Pi );
    _nb_param = _vlmtree->nb_leaves() * (_size-1);  
    
    delete _vlmtree;
  }

  ~VLMarkov(){}
};
#endif
