 /* seqpp/pmm_forest.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


/*!
  \internal
  \file pmm_forest.h
  \author V.Miele-P.Y.Bourguignon
  \brief Header for the random parsimonious Markov tree class
*/


#ifndef SEQPP_PMM_FOREST_H
#define SEQPP_PMM_FOREST_H

#include <seqpp/pmm_tree.h>

/*!
  \internal
  \class pmm_forest seqpp/pmm_forest.h
  \brief forest of pmm_tree
*/
class pmm_forest  {

 public:
  /*!
    \brief Constructor
    \param part Associated partition (must be alive until the destruction of the pmm_tree)  
    \param alphabet_size size of the alphabet
    \param depth The depth of the tree to build
    \param prior_alpha alpha for the a priori law, one value per alphabet element
    \param motif_prior activates a weight-function on the priors, weight proportionnal on each motif in the tree 
    \param penalty penalty
  */
  pmm_forest( Partition & part,
	      short alphabet_size, 
	      int depth, 
	      vector<double>& prior_alpha, bool motif_prior = true,
	      double penalty = 0. );

  //! Destructor
  ~pmm_forest();

   /*!
    \brief Performs maximal a posteriori selection
    \param count is the word count
    \param decal_required necessary when using a count of word from 1-word to (_max_depth+1)-word
  */
  pmm_tree select( const unsigned long * count );
  
  
  //! generate the extended tree from a count
  /*!
    \param count associated count; if NULL, the current count is used
  */
  bool extended_tree( const unsigned long * count );

  //! generate the extended tree from a the current count
  bool extended_tree(){
    return extended_tree(NULL);
  }

  bool extended_ok() const{
    return (_ftree!=NULL);
  }

  //! compute the mean posterior likelihood over the parameters and the trees
  /*!
    WARNING: To use after a extended_tree call
  */
  double mean_post_log_likelihood( const unsigned long * count_e );

  //! compute the mean posterior likelihood over the parameters and the trees
  /*!
    WARNING: To use after a extended_tree call
  */
  double mean_post_log_likelihood( );

  //! 

  //! draw a pmm_tree from the extended tree
  /*!
    WARNING: To use after a extended_tree call
  */
  pmm_tree draw( gsl_rng * r );

  //! compute the mean posterior likelihood over the parameters
  double post_log_likelihood( const unsigned long * count_e );
  
  double post_log_likelihood_2( const unsigned long * count_e );


  void set_penalty( double penalty ){
      _penalty = penalty;
  }

  //! returns the current pmm_tree for a given phase
  pmm_tree& current_pmm_tree( ){
    return _curr_p_t;
  }

 protected:

  // Major members --------------------------------

  //! associated partition
  Partition _part;
  //! number of possibilities in the partition
  long _part_nbposs;

  //! alphabet size 
  short _alphabet_size;
  //! synonyms alphabet size
  short _syn_size;
  //! tree depth
  int _depth;


  //! alpha for the prior law
  vector<double> _prior_alpha;
  vector<double> _tmp_prior_alpha;
  bool _motif_prior;

  //! penalty on the leaves number
  double _penalty;

  //! current pmmm_tree
  pmm_tree _curr_p_t;

  //! check if complete method is called in select
  bool _completed;

  
  // Optimization Tools ---------------------------- 
  short * _corresp;
  //! code("A", i) = code("A") en position i sur depth+1 posit
  int ** _code;
  //! count des depth+1-mots par ordre alphabetique
  vector<unsigned long> _count;
  int * _codeinterm;
  Partition::const_iterator _it;
  Partition::const_iterator _end;
  Partition::const_iterator _bestpart;
  vector<short>::const_iterator * _tab;
  vector<short>::const_iterator * _tabend;

  // Mean&extended tools --------------------------------------
  /*! 
    array representation of the extended tree
    indexed by codes in base _nbposs
  */
  double * _ftree; 
  double _sum_ftree; 
  double * _ftree_mlike;
  double * _ftree_to_extend;
  //! prior alpha value for each transition
  double ** _alpha;
  //! used in draw
  double* _theta;
 //! number of possible elements in a Partition
  long _nbposs; 
  /*! 
    code the path in base _nbposs in growing order
    <0> is 0, <1> is 1,...., <0-0> is 15, <0-1> is 16,..,<15-15> is 239, <0-0-0> is 240,....<15-15-15> is 3614 for example with _nbposs=15
  */
  Coder * _path;
  /*! 
    current path in the tree
    <0>, <1>,...., <0-0>, <0-1>,..,<15-15>, <0-0-0>,....<15-15-15> for example with _nbposs=15
  */     
  vector<int> _vpath;
  //! current code path
  long _cpath;



  // Private Methods --------------------------------
  /*!
    \brief Counts the motif given by path in the specified count, with the next alphabet_size chars as last letter
    \param path the motif to count 
    \param counts vector of count for each next letter
  */
  long count_hmotif( vector< int > & path, 
		     vector<long> & counts, short howmany_next ) ;
  
  double select_r( pmm_tree_rep& rep, node_base& node ) ;
  

  double extended_tree_core( const unsigned long * count, bool clear_count=true );
  //! recursive generation of the extended tree
  double extended_tree_r( short depth );
  
  //! recursive step of the draw method
  void draw_r( node_base& node, short depth, gsl_rng * r );
  
  //! recursive step of mean_posterior_likelihood
  double mean_post_likelihood_r( short depth );
  
  //! draws the transition parameters
  void draw_dist(double * alpha, gsl_rng * r) ;

  //! computes the log of the sum of the exp of the logs
  double ln_sumexp(const double * logs, int nlogs) ;


  double post_log_likelihood_r( const unsigned long * count_e,
				node_base& node,  bool force=false );

  //! return the number of words concerned by path
  int count_hmotif( vector< int > & path, 
		    vector<long> & counts_t,
		    const unsigned long * count_e, vector<long> & counts_e, 
		    short howmany_next );
    

 //---------------------------------------------------------
  void set_code();

  void set_count( const unsigned long * count,  bool clear_count=true );
  void set_count_r( const unsigned long * count, 
		    unsigned long code, unsigned long codesyn, short depth );

  //---------------------------------------------------------
 //! compute the number of word associated to a path
  int nbword_in_vmotif( const vector< int > & path );
  //! number of word concerned by a coded path
  int nbword_in_cmotif( long cpath );
};
  //----------------------
#endif //PMM_FOREST
