/* seqpp/pmm_tree.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


/*!
  \internal
  \file pmm_tree.h
  \author P.-Y. Bourguignon-V.Miele-M.Hoebeke
  \brief Header for the parsimonious Markov tree class
*/


#ifndef SEQPP_PMM_TREE_H
#define SEQPP_PMM_TREE_H

#include <seqpp/pmm_node.h>
#include <seqpp/pmm_leaf.h>
#include <seqpp/SequenceSet.h>
#include <seqpp/Partition.h>
#include <string>
#include <gsl/gsl_sf.h>

#ifdef HAVE_LIBXML2
#include <libxml/tree.h>
#endif
using namespace std ;


/*!
  \internal
  \class pmm_tree seqpp/pmm_tree.h
  \brief Representation of pmm_tree
*/
class pmm_tree_rep{
 public:

  /*!
    \brief Empty Constructor
  */
  pmm_tree_rep( ){
    _countref=1;_alphabet_size=0;_syn_size=0;
    _root=NULL;_depth=0;_nbleaves=0;_postloglike=PRECISION;
  }

  /*!
    \brief Constructor
    Only builds the root
    \param part Associated partition (must be alive until the destruction of the pmm_tree_rep)
    \param alphabet_size size of the alphabet
    \param depth The depth of the tree to build
  */
  pmm_tree_rep( Partition& part, short alphabet_size, int depth );

  pmm_tree_rep(const pmm_tree_rep & rep){
    cout<<"bad constr copy call"<<endl;
  }

  //! Destructor
  virtual ~pmm_tree_rep();   

  /*!
   * \brief Deduces the markov matrix (stored in line) from the pmm
   */
  void tree_to_matrix( double* mat );
   

#ifdef HAVE_LIBXML2  
  void save( const Translator & trans, 
	     xmlNodePtr parent_node, const string &treename );
#endif
  
  //! return size
  short tell_alphabet_size () const{
    return _alphabet_size;
  }
  //! returns partition
  const Partition& get_partition() const{
    return _part;
  }

  //! returns the current number of leaves
  int nb_leaves() const{return _nbleaves;}
  //! count a added leaf
  void add_leaf(){_nbleaves++;}
  //! count a cutted leaf
  void cut_leaf(){_nbleaves--;}
  //! complete the tree
  double complete(node_base& node) ;      
  //! prune the tree under n
  void prune(pmm_node& n); 
  //! returns the root
  pmm_node & getRoot() const{return *_root;}
  //! return the leaf
  pmm_leaf & getLeaf(vector<short> & rpath) const;  
  //! return the leaf for a syn_size-base code
  pmm_leaf & getLeaf(unsigned long code) const;
  //! return depth
  short tellDepth() const{
    return _depth;
  }

   //! returns the number of reference 
  short countref() const{
    return _countref;
  }
  //! postfixed
  pmm_tree_rep& operator ++ (int) {
    _countref++;
    return *this;
  } //! postfixed
  pmm_tree_rep& operator -- (int) {
    _countref--;
    return *this;
  }


  //! set the postloglike of the tree
  void set_postloglike( double ptl ){_postloglike = ptl;}
  //! increment the postloglike of the tree
  void add_postloglike( double ptl ){_postloglike += ptl;}
  //! return the postloglike of the tree
  double tell_postloglike(){return _postloglike;}

  //! return the tree prior
  double tell_tree_prior(){return _curr_tree_prior;}
  //! set the tree prior
  void set_tree_prior( double x ){_curr_tree_prior = x;}
  //! add to the tree prior
  void add_tree_prior( double x ){_curr_tree_prior += x;}
  
 protected:
  //! number of reference pmm_tree on this representation pmm_tree_rep
  short _countref;
      
  //! associated partition
  Partition _part;  
  short _alphabet_size;
  short _syn_size;
  pmm_node * _root ;
  //! Depth of the tree
  int _depth;
  //! Number of leaves
  int _nbleaves;

  //! postloglike of the parameters
  double _postloglike;
  double _curr_tree_prior;
};

/*!
  \internal
  \class pmm_tree seqpp/pmm_tree.h
  \brief estimation tree for Parcimonious Markov model selection 
*/
class pmm_tree{
  pmm_tree_rep * _rep;
 public:
  pmm_tree(){
    _rep =  new pmm_tree_rep();
  }
  pmm_tree(Partition& part, short alphabet_size, int depth){
    _rep = new pmm_tree_rep(part, alphabet_size, depth);
  }
  pmm_tree(const pmm_tree& p_t){
    _rep = p_t._rep;
    (*_rep)++;
  }
  pmm_tree(pmm_tree_rep& rep){
    _rep = &rep;
    (*_rep)++;
  }
  ~pmm_tree(){
    if (_rep->countref()>1)
      (*_rep)--;
    else
      delete _rep;
  }
  pmm_tree& operator=(pmm_tree & p_t){
    if (this != &p_t){
      // delete
      if (_rep->countref()>1)
	(*_rep)--;
      else
	delete _rep;
      // new
      _rep = p_t._rep;
      (*_rep)++;
    }
    return (*this);
  } 
  short tellDepth() const{
    return _rep->tellDepth();
  }

  /*!
   * \brief Deduces the markov matrix (stored in line) from the pmm
   */
  void tree_to_matrix( double* mat ){
    _rep->tree_to_matrix( mat );
  }
#ifdef HAVE_LIBXML2  
  void save( const Translator & trans, 
	     xmlNodePtr parent_node, const string &treename ){
    _rep->save( trans, parent_node, treename );
  }
#endif
  //! returns the current number of leaves
  int nb_leaves() const{return _rep->nb_leaves();}

  //! returns the root
  pmm_node & getRoot() const{return _rep->getRoot();}

  //! set the postloglike of the tree
  void set_postloglike( double ptl ){_rep->set_postloglike(ptl);}
  //! increment the postloglike of the tree
  void add_postloglike( double ptl ){_rep->add_postloglike(ptl);}
  //! return the postloglike of the tree
  double tell_postloglike(){return _rep->tell_postloglike();}
  //! return the tree prior
  double tell_tree_prior(){return _rep->tell_tree_prior();}
  //! return the tree prior
  void set_tree_prior( double x ){_rep->set_tree_prior(x);}

  
  class const_iterator{
  private :
    const pmm_tree_rep * _rep;
    node_base * _curr_node;
    // current list of non visited sons
    vector<vector<node_base *> *> _curr_non_visited_sons_path;
    // index in the list of the current non visited son
    vector<int> _curr_index_in_path;    

  protected :
    friend class pmm_tree;
    const_iterator( const pmm_tree& p_t, node_base* curr_node ){
      _rep=p_t._rep; _curr_node=curr_node;
      
      //_curr_non_visited_sons_path.assign(1+_rep->tellDepth(),NULL);
      //_curr_index_in_path.assign(_rep->tellDepth(),-1);

      short d = _rep->tellDepth();
      for (short i=0; i<d+1; i++){
	_curr_non_visited_sons_path.push_back(NULL);
	_curr_index_in_path.push_back(-1);
      }
    }
    const_iterator( const pmm_tree_rep& rep, node_base* curr_node ){
      _rep=&rep; _curr_node=curr_node;
      short d = _rep->tellDepth();
      for (short i=0; i<d+1; i++){
	_curr_non_visited_sons_path.push_back(NULL);
	_curr_index_in_path.push_back(-1);
      }
    }
  public:  
    const_iterator( ){
      _rep=NULL;
      _curr_node=NULL;
    }
    const_iterator( const const_iterator & it ){
      _rep=it._rep;_curr_node=it._curr_node;
      _curr_non_visited_sons_path=it._curr_non_visited_sons_path;
      _curr_index_in_path=it._curr_index_in_path;      
    }

    bool operator != ( const const_iterator & it ) const{
      return ((_rep!=it._rep)||(_curr_node!=it._curr_node));
    }
    bool operator == ( const const_iterator & it ) const{
      return ((_rep==it._rep)&&(_curr_node==it._curr_node));
    }
    const_iterator& operator = (  const const_iterator & it ){
      if (  it != *this ){
	_rep=it._rep;
	_curr_node=it._curr_node;
	_curr_non_visited_sons_path=it._curr_non_visited_sons_path;
	_curr_index_in_path=it._curr_index_in_path;
      }
      return *this;
    }
    bool operator++(int);      
    
    pmm_leaf & operator * () const{
      return  *(dynamic_cast<pmm_leaf *>(_curr_node));
    }
  };
  
  const_iterator begin() const{
    return const_iterator( *(this->_rep), &(this->getRoot()) );
  }
  
  const_iterator end() const{
    return const_iterator( *(this->_rep), NULL );
  } 
};
#endif
