/* seqpp/tree_t.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


/*!
  \internal
  \file tree_t.h
  \brief Header file for the tree<T> template class.

  This file implements a tree template with iterator.
  \date Oct. 2003
  \author P.-Y. Bourguignon 
*/


#ifndef SEQPP_TREE_T_H
#define SEQPP_TREE_T_H

#include <iostream>
#include <cstdlib>


using namespace std;


/*!
  \internal
  \class tree_t seqpp/tree_t.h
  \brief tree template class

This class is not used as is, but derived to a tree_node<T> class, so that the user does not see it.
*/

class tree_node_base
{
      public:
	tree_node_base * *_sons;	//!< An array of pointers on the sons of the node
	tree_node_base *
		_father;	//!< A pointer on the father of the node
	short
		_label;		//!< The label of the node (it's index in its father's _sons array)
	short
		_size;		//!< The maximal number of edges down the node

	/*!
	 * \brief Constructor
	 * \param size The (maximal) number of edges down the node
	 * \param label The label of the node (must be equal to its index in its father's _sons array
	 * \param father A pointer to the father of the node to be built
	 */
	inline tree_node_base (short size, short label, tree_node_base * father);
	//! Copy constructor
	/*!
	 * Note it is recursive : the copy of one node implies the copy of the whole subtree under tree.
	 */
	inline tree_node_base (tree_node_base & source);
	//! Destructor
	inline virtual ~ tree_node_base ();
	/*!
	 * \brief Affectation operator
	 * 
	 * \warning Do not use this method, it's just there to provide stuff to the compiler.
	 */
	inline tree_node_base & operator= (tree_node_base & source);
};


tree_node_base::tree_node_base (short size, short label =-1, tree_node_base * father = NULL)
{
	if (label != -1)
	{
		_label = label;
		_father = father;
	}
	else
		_father = NULL;
	_size = size;
	_sons = new tree_node_base *[_size];
	for (short label = 0; label < _size; label++)
	{
		_sons[label] = NULL;
	}
};

tree_node_base::tree_node_base (tree_node_base & source)
{
	_father = source._father;
	_size = source._size;
	_sons = new tree_node_base *[_size];
	for (int i = 0; i < _size; i++)
	{
		if (source._sons[i])
			_sons[i] = new tree_node_base (*(source._sons[i]));
		else{
		  _sons[i] = NULL;
		}
	}
};

tree_node_base::~tree_node_base ()
{
	for (short label = 0; label < _size; label++)
	{
		if (_sons[label])
			delete _sons[label];
	}
	delete[]_sons;
};

tree_node_base & tree_node_base::operator= (tree_node_base & source)
{
	cerr << "Error in _tree_node_base : no affectation operator in clas"
		<< endl;
	exit (1);
};


/*!
  \internal
  \brief The nodes of the tree_t class
*/

template < class T > class tree_node:public tree_node_base
{
      public:
	T _content;		//!< The content of the node, of user-defined type T
	//! Constructor
      tree_node (short size, T & init, short label = 0, tree_node_base * father = NULL):tree_node_base (size, label, father),
		_content
		(init)
	{
	};
	//! Copy constructor
      tree_node (tree_node & source):tree_node_base (source)
	{
		_content = source._content;
	};
};

//###########################################################################

/*!
  \internal
  \brief The base class for iterators
*/

class tree_iterator_base
{
      public:
	tree_node_base * _current;	//!< A pointer to currently pointed node

	//! Constructor with void initialization
	inline tree_iterator_base ();
	//! Constructor with initialization on the node pointed by ptr
	inline tree_iterator_base (tree_node_base * ptr);
	// tree_iterator_base( tree_iterator_base & source) using default
	// tree_iterator_base & operator=(tree_iterator_base & source) using default
	/*!
	 * \brief Moves to the son indexed by label 
	 * \param label the index of the son to go to in the current node's _sons array
	 */
	inline void
	son (short label);
	/*!
	 * \brief Moves to the father node
	 */
	inline void father ();

	inline tree_node_base * tell_father();
	/*
	 * \brief Returns the label of the currently pointed node
	 */
	inline short
	label () const;
	//! Returns true if the current node has no son index by label, false otherwise
	inline bool on_leaf (short label) const;
	//! Returns true if the current node has no son at all, false otherwise.
	inline bool on_leaf ()const;
	inline bool
	on_leaf_one () const;
	//! Returns true if the currently pointed node is the root
	inline bool on_root ()const;
	/*!
	 * \brief Operator to iterate through the tree
	 * 
	 * Iteration in the tree is done first vertically, from root to leaves, and second horizontally
	 */
	inline tree_iterator_base & operator++ ();
	/*! 
	 * \brief Iterates through the tree in reverse order
	 * 
	 * Made so that (it++)-- results in no modification.
	 */
	inline tree_iterator_base & operator-- ();
	//! Comparison operator
	inline bool operator== (const tree_iterator_base & source) const;
	//! Comparison operator
	inline bool operator!= (const tree_iterator_base & source) const;
	//! Returns the depth of the node pointed by iterator
	inline int
	level () const;
};

tree_iterator_base::tree_iterator_base ()
{
	_current = NULL;
};

tree_iterator_base::tree_iterator_base (tree_node_base * ptr)
{
	_current = ptr;
};

void
tree_iterator_base::son (short label)
{
	_current = _current->_sons[label];
};

void
tree_iterator_base::father ()
{
	_current = _current->_father;
};

tree_node_base *
tree_iterator_base::tell_father ()
{
  return _current->_father;
};

short
tree_iterator_base::label () const
{
	return (_current->_label);
};

bool tree_iterator_base::on_leaf (short label) const
{
	return bool (!(_current->_sons[label]));
};

bool tree_iterator_base::on_leaf () const
{
	bool
		test = true;
	for (short label = 0; label < _current->_size; label++)
		test = test & !(bool (_current->_sons[label]));
	return test;
};

bool tree_iterator_base::on_leaf_one () const
{
	bool
		test = false;
	for (int i = 0; i < _current->_size; i++)
	{
		if (!(_current->_sons[i]))
			return true;
	}
	return test;
};

bool tree_iterator_base::on_root () const
{
	return (_current->_father == NULL);
};

tree_iterator_base & tree_iterator_base::operator++ ()
{
	bool goon = true;
	for (int i = 0; i < _current->_size; i++)
	{
		if (_current->_sons[i])
		{
			_current = _current->_sons[i];
			goon = false;
			break;
		}
	}
	while (goon && bool (_current->_father))
	{
		short
			label = _current->_label;
		for (int i = label + 1; i < _current->_size; i++)
		{
			if (_current->_father->_sons[i])
			{
				goon = false;
				_current = _current->_father->_sons[i];
				break;
			}
		}
		if (goon)
		{
			_current = _current->_father;
		}
	}
	return *this;
};

tree_iterator_base & tree_iterator_base::operator-- ()
{
	short
		label;
	if (_current->_father != NULL)
	{
		_current = _current->_father;
		label = _current->_label;
	}
	else
		label = _current->_size;

	bool goon = false;
	for (int i = label - 1; i > -1; i--)
	{
		if (_current->_sons[i])
		{
			_current = _current->_sons[i];
			goon = true;
			break;
		}
	}
	if (goon == false)
		return *this;
	while (goon)
	{
		goon = false;
		for (int i = _current->_size - 1; i > -1; i--)
		{
			if (_current->_sons[i])
			{
				_current = _current->_sons[i];
				goon = true;
				break;
			}
		}
	}
	return *this;
};

bool tree_iterator_base::operator== (const tree_iterator_base & source) const
{
	return _current == source._current;
};

bool tree_iterator_base::operator!= (const tree_iterator_base & source) const
{
	return _current != source._current;
};

int
tree_iterator_base::level () const
{
	int level = 0;
	tree_iterator_base it = *this;
	while (!it.on_root ())
	{
		it.father ();
		level++;
	}
	return level;
};

//###########################################################################
/*!
  \internal
  \brief The iterator class for the tree_t data structure.
*/
template < class T, class TRef, class TPtr > class tree_iterator:public tree_iterator_base
{
      public:
	typedef tree_iterator < T, T &, T * >iterator;	//!< The iterator suitable for handling data of type T
	typedef tree_iterator < T, const T &, const T *>const_iterator;	//!< Iterator giving const access to the data in the nodes
	typedef tree_iterator < T, TRef, TPtr > _self;

	typedef T value_type;	//!< Type of the data
	typedef TRef reference;	//!< Reference on the data
	typedef TPtr pointer;	//!< Pointer on the data
	typedef tree_node < T > _node;	//!< The node type

	/*!
	 * \brief Constructor
	 * \param source The node pointed by the new iterator
	 */
      tree_iterator (_node * source):tree_iterator_base (source)
	{
	};
	/*!
	 * \brief Constructor
	 * \overload
	 */
      tree_iterator (_node & source):tree_iterator_base (&source)
	{
	};
	/*!
	 * \brief Constructor
	 * 
	 * Iterator is created without initialization of its pointer.
	 */
      tree_iterator ():tree_iterator_base ()
	{
	};
	/*!
	 * \brief Copy constructor
	 */
	tree_iterator (const iterator & it):tree_iterator_base (it._current)
	{
	  //cout<<"copy construct"<<endl;
	};
	/*!
	 * \brief Dereference operator
	 * 
	 * Provides transparent acces to data in a node pointed by iterator it using (*it)
	 * \warning It is defined as constant for the iterator, because it does not imply any movement of the iterator, nor node deletion or insertion.
	 */
	inline reference operator* () const
	{
		return static_cast < _node * >(_current)->_content;
	};
	/*!
	 * \brief Dereference operator
	 * \warning It is defined as constant for the iterator, because it does not imply any movement of the iterator, nor node deletion or insertion.
	 * 
	 * Provides transparent acces to data in a node pointed by iterator it using it->...
	 */
	inline pointer operator-> () const
	{
		return &(operator* ());
	};
	/*!
	 * \brief Inserts a leaf in the tree
	 * \param label The index of the leaf to be created
	 * \param init value used for node's data initialization
	 * 
	 * Note it moves the iterator on the created son.
	 */
	inline iterator & insert (short label, value_type & init)
	{
	  if (_current->_sons[label]){
	    delete _current->_sons[label];
	  }
	  _current = (_current->_sons[label] =
		      new _node (_current->_size,
				 init, label, _current));
	  return *this;
	};
	/*!
	 * \brief Removes the currently pointed node
	 * 
	 * Note the iterator will point on the father node after the deletion. That's why it cannot be called while pointing on the root node, which can't be deleted at all.
	 */
	inline iterator & remove ()
	{
		short label = _current->_label;
		if (_current->_father)
		{
			_current = _current->_father;
			delete _current->_sons[label];
			_current->_sons[label] = NULL;
		}
		else
			cerr << "tree_iterator<T>::remove : Root node can't be removed" << endl;
		return *this;
	};	
};






/*!
  \internal
  \brief The tree_t template class
  
  This class implements a tree structured data storage. Through a template mechanism, it allows data of any type. It provides its own iterator type, providing methods to go through the tree without repetition and quite quickly.
*/
template < class T > class tree_t
{
      public:
	typedef tree_node < T > _node;
	typedef tree_iterator < T, T &, T * >iterator;	//!< The iterator type
	typedef tree_iterator < T, const T &, const T *>const_iterator;	//!< The iterator type, providing constant references on the data.
	typedef T value_type;
	typedef T & reference;
	typedef T *pointer;

      protected:
	_node * _root;		//!< The root node of the tree
	/*!
	 * \brief Recursive method used to expand the tree to a chosen depth.
	 */
	inline void
		filltree (int size, value_type & init,
			  int depth, iterator & it)
	{
		for (int i = 0; i < size; i++)
		{
			it = it.insert (i, init);
			if (depth > 1)
			{
				filltree (size, init, depth - 1, it);
			}
			if (it._current->_father)
				it.father ();
		}
	};
      public:
	/*!
	 * \brief Constructor
	 * \param size the maximal number of edges down a node of the tree
	 */
	tree_t (int size)
	{
		_root = new _node (size);
	};
	/*!
	 * \brief Constructor with root's data initialization
	 * \overload
	 * \param init The value used to initialize root's data
	 */
	tree_t (int size, value_type & init)
	{
		_root = new _node (size, init, -1, NULL);
	};
	/*!
	 * \brief Constructor with data initialization and initial depth
	 * \param depth the initial depth of the tree
	 * \overload
	 */
	tree_t (int size, value_type & init, int depth)
	{
		_root = new _node (size, init);
		iterator it = begin ();
		filltree (size, init, depth, it);
	};
	//! Copy constructor
	tree_t (tree_t & source)
	{
		cerr << "tree_t : copy constructor not yet implemented" <<
			endl;
	};
	//! Destructor
	virtual ~tree_t ()
	{
		delete _root;
	};
	//! Returns an iterator initialized on the root of the tree
	iterator begin () const
	{
		return iterator (_root);
	};
	/*
	//! Returns an iterator initialized on the node at address path
	iterator nodes (path_t path) const
	{
		iterator it = begin ();
		for (int i = 0; i < path._length; i++)
		{
			it.son (path._path[i]);
		}
		return it;
	};
	//! Returns an iterator on the leaf found following the path
	iterator leaves (path_t path) const
	{
		iterator it = begin ();
		int step = 0;
		while ((step < path._length) && !(it.on_leaf (path[step])))
			  it.son (path[step++]);
		  return it;
	};
	//! Returns a const_iterator initialized on the node at path
	const_iterator const_nodes (path_t path)
	{
		const_iterator it = begin ();
		for (int i = 0; i < path._length; i++)
		{
			it.son (path._path[i]);
		}
		return it;
	};
	//! Returns the path of the node pointed by iterator
	path_t & path (iterator it, path_t & _path)
	{
		int i = it.level ();
		if (_path._length != i)
		{
			cerr << "Error in tree_t::path : provided path_t has not required length" << endl;
			return _path;
		}

		while (!it.on_root ())
		{
			_path[--i] = it.label ();
			it.father ();
		}
		return _path;
	};
	*/
};

#endif
