
/* seqpp/vlm_tree.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


/*!
  \internal
  \file vlm_tree.h
  \brief Header file for the vlm_tree library.

  This file implements context trees for biological sequences modelling
  \date Oct. 2003
  \author P.-Y. Bourguignon <A HREF="mailto:bourguignon@genopole.cnrs.fr"> bourguignon@genopole.cnrs.fr </A>
*/


#ifndef SEQPP_VLM_TREE_H
#define SEQPP_VLM_TREE_H

#include <seqpp/tree_t.h>
#include <seqpp/SequenceSet.h>
#include <seqpp/Coder.h>

/*!
  \internal
  \brief A class implementing context trees
  
  This class implements variable length markov chains models, allowing estimation, goodness of fit evaluation, and classical estimation. 
*/

class vlm_tree:public tree_t < vector<double> >
{ 
 public:
  typedef tree_iterator < vector<double>, vector<double> &, vector<double> * >iterator;	//!< Iterator on the context tree
  typedef tree_t < vector<double> > tree;	//!< Tree
  typedef tree_node < vector<double> > node;	//!< Nodes


  /*!-------------------------------------------------------------
   * \brief Constructor
   * 
   * As inherited of the base tree_t class
   */
  vlm_tree (Coder & coder, 
	  int size, int maxdepth, vector<double> & init)
    :tree( size, init,	maxdepth )
  {      
    _size = size;
    _nbleaves = 0;
    _coder = &coder;
    _max_depth = maxdepth;
    _tmpcounts = new long[_size];
  };

  /*!-------------------------------------------------------------
   * \brief Constructor
   * \overload
   * Only builds nodes associated to words appearing at least min_count times in the sequence
   */
  vlm_tree (Coder & coder,
	  int size, vector<double> & init, int maxdepth, 
	  const unsigned long * _count, int min_count = 3)
    :tree (size, init)
  {
    _size = size;
    _nbleaves = 0;
    _coder = &coder;
    _max_depth = maxdepth;
    _tmpcounts = new long[_size];
    iterator it = begin ();
    set_count( _count );
    filltree (size, init, 0, it, min_count);
  };
  
  //! destructor
  virtual ~vlm_tree(){
    delete[] _tmpcounts;
  }

  /*!-------------------------------------------------------------
    \brief Estimates the context tree using Context method and returns the nber of leaves
	  
    This method is recursive, and performs tree estimation according to the context algorithm.
    \param count The CountTree used to calculate transitions and evaluate pruning criterion.
    \param The cutoff value used in the context algorithm
  */
  long estimate_context ( const unsigned long * count, double cutoff );

  /*!-------------------------------------------------------------
   * \brief Likelihood evalutation
   * \param from a word count
   */
  double loglikelihood( const unsigned long * count );
  
  /*!-------------------------------------------------------------
   * \brief Returns the real depth of the tree
   */
  int depth () const;

  /*!-------------------------------------------------------------
   * \brief Returns the current number of leaves
   */
  int nb_leaves() const{
    return _nbleaves;
  }
  
  /*!-------------------------------------------------------------
   * \brief Deduces the markov matrix (stored in line) from the tree
   */ 
  void tree_to_matrix( double* mat );

  // on verra + tard
  //void fig_output (char *FigFile);

  /*!-------------------------------------------------------------
   * \brief Returns the proba P(next|word)
   * \param word An array of short describing the context
   * \param length The length of the context
   * \param next The symbol to be drawn
   */
  //double proba(const vector<short> word, short next) const;
  
  /*!-------------------------------------------------------------
   * \brief Likelihood evalutation
   * \param count A CountTree to get the counts from
   */
  /*
    double likelihood (const SequenceSet & seq);
  */

  /*
    double BIC (const SequenceSet & seq, double cutoof){ 
    nb_leaves =	arbre.estimate_context( arrseq, cutoff );
    return -2*likelihood(seq)
    +2*(_size-1)*nb_leaves*log( double(seq.tell_total_length()) );
    }
  */
  
 private:
  int _max_depth;		//!< The maximal depth of the tree
  int _size;		//!< The cardinal of the alphabet
  int _nbleaves;        //!< Number of leaves

  //! count of 1-words to (_max_depth+1)-words
  const unsigned long * _count;

  // tools
  long *_tmpcounts;
  Coder * _coder;


  /*!-------------------------------------------------------------
   * \brief Expands the tree to the required depth
   * \overload
   * This method performs tree expansion
   */ 
  void filltree (int size, vector<double> & init, int depth,
		 iterator & it,  int min_count = 3);
  
  /*!-------------------------------------------------------------
   * \brief Recursive step of the transitions  method
   * \overload
   * \param it an iterator pointing on the node whose transitions are to be calculated
   */
  void transitions ( iterator & it);
  
  
  /*!-------------------------------------------------------------
   * \brief Recursive step of the estimate_context(CountTree & count) method.
   * \param it iterator pointing on the node to be estimated at this step
   * \overload
   */
  bool estimate_context ( double &cutoff,
			  vlm_tree::iterator & it);
 
  /*!-------------------------------------------------------------
   * \brief Learn all transitions of the tree
   * 
   * Allocates and initializes all multinomial laws in the tree
   * \param count The CountTree used to calculate transitions
   */
  void transitions( );

 
  /*!-------------------------------------------------------------
   * \brief Learn all transitions of the tree
   * 
   * Allocates and initializes all multinomial laws in the tree
   * \param count The CountTree used to calculate transitions
  */
  long estimate_leaves( );

  //! 
  void set_count( const unsigned long * count ){    
    _count = count; 
  }
 
  //! return the code associated to the possition in the tree
  long path_code( iterator & it )
  {
    iterator tmpit( it ); // a netoyer!!!!!
    _coder->clear();
    while (!tmpit.on_root ())
      {
	_coder->push_back( tmpit.label () );
	tmpit.father ();
      }
    return _coder->_code; 
  }
};
#endif
